#!/usr/bin/env python
"""
This class will run various validation tests and gather information
about Report Objects that are cluster nodes.

@author    :  Shane Bradley
@contact   :  sbradley@redhat.com
@version   :  2.03
@copyright :  GPLv2
"""
import string
import logging
import os
import re

import sx
import sx.plugins
from sx.logwriter import LogWriter

from sx.plugins.lib.clusternodes import ClusterNodes
from sx.plugins.lib.clusternode import ClusterNode
from sx.plugins.lib.clusterconfevaluator import ClusterConfEvaluator

from sx.reports.sosreport import Sosreport
from sx.reports.sysreport import Sysreport

class Cluster(sx.plugins.PluginBase):
    """
    This class will run various validation tests and gather
    information about Report Objects that are cluster nodes.
    """
    def __init__(self, pathToPluginReportDir="") :
        """
        This init takes the root path to where the reports will be
        written. The parent class will then create the correct
        directory structure for the plugin.

        @param pathToPluginReportDir: This is the root path to where
        the report files will be written.
        @type pathToPluginReportDir: String
        """
        sx.plugins.PluginBase.__init__(self, "Cluster",
                                       "This plugin will verify installation and configuartion of the cluster.",
                                       ["Sosreport", "Sysreport"], True, True, {}, pathToPluginReportDir)

        self.__cnc = ClusterNodes()

    # #######################################################################
    # Report functions
    # #######################################################################
    def __getClusterNodesSystemInformation(self) :
        """
        Returns a string that contains information about the system
        information for each node.

        @return: A string that contains information about the system
        information for each node.
        @rtype: String
        """
        rString  = ""
        for clusternode in self.__cnc.getClusterNodes():
            if (len(rString) > 0):
                rString += "\n"
            unameASplit = clusternode.getUnameA().split()
            unameA = ""
            for i in range (0, len(unameASplit)):
                if (i == 4) :
                    unameA += "\n\t      "
                unameA += "%s " %(unameASplit[i])
                i = i + 1
            rString += "Hostname:     %s" %(clusternode.getHostname())
            rString += "\nRH Release:   %s" %(clusternode.getDistroRelease())
            rString += "\nUptime:       %s" %(clusternode.getUptime())
            rString += "\nUname -a:     %s" %(unameA)
            rString += "\n%s" %(str (clusternode))
        return rString

    def __getClusterNodesPackagesInstalled(self) :
        """
        Returns a string that contains information about the
        cluster/cluster-storage packages installed for each node.

        @return: A string that contains information about the
        cluster/cluster-storage package installed for each node.
        @rtype: String
        """
        rString  = ""
        for clusternode in self.__cnc.getClusterNodes():
            if (len(rString) > 0):
                rString += "\n"
            rString += "%s:" %(clusternode.getHostname())

            # Verify cluster packages
            packages = clusternode.getClusterPackagesVersion()
            keys = packages.keys()
            keys.sort()
            if (not len(keys) > 0) :
                rString += "\n\t      No Cluster Packages Found" 
            for key in keys:
                cPackages = packages[key]
                cPackages.sort()
                for cPackage in cPackages:
                    rString += "\n\t      %s" %(cPackage)
            rString += ("\n")

            # Verify cluster-storage package
            packages = clusternode.getClusterModulePackagesVersion()
            keys = packages.keys()
            keys.sort()
            if (not len(keys) > 0) :
                rString += "\n\t      No GFS2/GFS Packages Found" 
            for key in keys:
                cPackages = packages[key]
                cPackages.sort()
                for cPackage in cPackages:
                    rString += "\n\t      %s" %(cPackage)
            rString += "\n"
        # Remove an extra newline
        rString = rString.rstrip("\n")
        return rString

    def __isClusterNodeNamesInHostsFile(self, clusternodeNames, networkMaps) :
        """
        This function returns True if all the node names that are in
        cluster.conf are in /etc/hosts.

        @return: Returns True if all the node names are in /etc/hosts.
        @rtype: Boolean

        @param clusternodeNames: The node names in the cluster.conf.
        @type clusternodeNames: Array
        @param networkMaps: A list of NetworkMaps for a clusternode.
        @type networkMaps: Array
        """
        # Iterate over ever nodename, we only need to search one
        # networkMap since all contain same mapping of /etc/hosts. If
        # 1 nodename is not found in /etc/hosts map  then
        # return False. 
        if (len(networkMaps) > 0):
            nmap = networkMaps[0]
            for nodename in clusternodeNames:
                if(not nmap.hasHostnameMapped(nodename)):
                    return False
            return True
        return False

    def __isClusterNodeNamesRFC2181Valid(self, clusterNodeNames) :
        """
        Returns True if the node names follow the RFC2181 standard.

        @return: Returns True if the node names follow the RFC2181
        standard.
        @rtype: Boolean

        @param clusterNodeNames: The node names in cluster.conf.
        @type clusterNodeNames: Array
        """
        for hostname in clusterNodeNames:
            if not re.match('^[a-zA-Z]([a-zA-Z0-9-]*[a-zA-Z0-9])?(\.[a-zA-Z]([a-zA-Z0-9-]*[a-zA-Z0-9])?)*$', hostname):
                return False
        return True

    def __doesGFS2ModuleRemoved(self, unameAData, packages):
        """
        In Red Hat Enterprise Linux 5.2, GFS2 was provided as a kernel
        module for evaluation purposes. In Red Hat Enterprise Linux
        5.3 GFS2 is now part of the kernel package. 

        If the Red Hat Enterprise Linux 5.2 GFS2 kernel modules have
        been installed they must be removed to use GFS2 in Red Hat
        Enterprise Linux 5.3. This function returns true if currently
        running kernel should have kmod-gfs* removed.
        
        @return: Returns True if kmod-gfs should be removed.
        @rtype: Boolean
        
        @param unameAData: A string containing uname data.
        @type unameAData: String
        @param packages: A list of packages.
        @type packages: Array
        """
        isGFSmoduleInstalled = False
        for package in packages:
            if (package.startswith("kmod-gfs2")) :
                isGFSmoduleInstalled = True
                break;
        unameSplit = unameAData.split()
        kernelVersionMinor = 0
        if (len(unameSplit) >= 2):
            kernelVersionMinor = unameSplit[2].split("-")[1].split(".")[0]

        return ((kernelVersionMinor > 128) and (isGFSmoduleInstalled))

    def __verifyServicesStatus(self, filename, chkconfigDict, statusEnabled=True):
        """
        This function writes the result of testing the status of a
        service in from the data from "chkconfig" command's data in a
        file.

        @param filename: This if filename where the results of tests
        will be written.
        @type filename: String
        @param chkconfigDict: A list of the status of various services.
        @type chkconfigDict: Dict
        @param statusEnabled: If True then it will report runlevels
        that are not enabled. If False then it will report on levels
        that are enabled.
        @type statusEnabled: boolean
        """
        if (not len(chkconfigDict) > 0):
            message = "There was no chkconfig data to check status of cluster services."
            logging.getLogger(sx.MAIN_LOGGER_NAME).error(message)
            return
        for cService in chkconfigDict:
            minRunLevel = 0
            maxRunLevel = 6
            # Get the min/max runlevels for current service
            for key in cService.keys():
                # [ {'cman': {3: False, 4: False, 5: False}} ....]
                serviceLevels =  cService[key]
                levels = serviceLevels.keys()
                if (not len(levels) > 0) :
                    continue
                minRunLevel = min(levels)
                maxRunLevel = max(levels)

            # Check each service
            for serviceName in cService.keys():
                cServiceLevel = cService[serviceName]
                if (not (len(cServiceLevel) > 0)):
                    # If there is no services
                    continue
                if statusEnabled:
                    # Checking that service is enabled on all
                    # runlevels. If num of keys is greater than zero than
                    # some service is disabled in the runlevel.
                    allEnabled = True
                    for level in cServiceLevel.keys() :
                        if (not cServiceLevel[level]):
                            allEnabled = False
                    message = "Testing if the service \"%s\" is enabled for runlevels %d-%d."%(key, minRunLevel, maxRunLevel)
                    self.writeTestResult(filename, message.rstrip(), (allEnabled))

                else:
                    # Checking that service is enabled on all
                    # runlevels. If num of keys is greater than zero than
                    # some service is disabled in the runlevel.
                    allDisabled = True
                    for level in cServiceLevel.keys() :
                        if (cServiceLevel[level]):
                            allDisabled = False
                    message = "Testing if the service \"%s\" is disabled for runlevels %d-%d."%(key, minRunLevel, maxRunLevel)
                    self.writeTestResult(filename, message.rstrip(), (allDisabled))


    # #######################################################################
    # Functions that should be overwritten in the plugin
    # #######################################################################
    def setup(self, reports) :
        """
        This function will setup data structure to hold any data/path
        to files that are needed to use in this plugin.

        @param reports: This is the list of Report Objects.
        @type reports: Array
        """
        message = "Running setup for plugin: %s" %(self.getName())
        logging.getLogger(sx.MAIN_LOGGER_NAME).log(LogWriter.STATUS_LEVEL, message)
        for report in reports:
            if (self.isValidReportType(report)) :
                self.__cnc.add(report)

    def report(self) :
        """
        This function will write to report files the results of the
        cluster validation tests and report any errors to console.
        """
        message = "Generating report for plugin: %s" %(self.getName())
        logging.getLogger(sx.MAIN_LOGGER_NAME).log(LogWriter.STATUS_LEVEL, message)
        message = "%s is generating a report of various information about the nodes in the cluster." %(self.getName())
        logging.getLogger(sx.MAIN_LOGGER_NAME).info(message)
        filename = "summary.txt"
        if (not len(self.__cnc.getClusterNodes()) > 0):
            message = "There were no cluster nodes found in these sosreport/sysreports"
            logging.getLogger(sx.MAIN_LOGGER_NAME).warn(message)
        else:
            # List of clusternodes in cluster.conf that do not have
            # corresponding sosreport/sysreport.
            missingNodesList = self.__cnc.listClusterNodesMissingReports()
            if (len(missingNodesList) > 0):
                message = "The following cluster nodes were did not have a report based on clusternode name:"
                for nodeName in missingNodesList:
                    message +="\n\t  %s" %(nodeName)
                logging.getLogger(sx.MAIN_LOGGER_NAME).warn(message)

                message = "The following cluster nodes were did not have a report:"
                for nodeName in missingNodesList:
                    message +="\n\t    %s" %(nodeName)
                self.write(filename, "%s\n\n" %(message))
            
            # ###################################################################
            # Summary of each node in collection
            # ###################################################################
            self.writeSeperator(filename, "Cluster Nodes Summary");
            result = self.__getClusterNodesSystemInformation()
            self.write(filename, result)
            self.writeSeperator(filename, "Cluster/Cluster-Storage Packages Installed");
            result = self.__getClusterNodesPackagesInstalled()
            self.write(filename, result)
            self.write(filename, "")

            # ###################################################################
            # Verify the global cluster configuration
            # ###################################################################
            self.writeSeperator(filename, "Cluster Configuration Summary");
            cce = ClusterConfEvaluator(self.__cnc.getBaseClusterConfFile())
            # This is list of messages that contain item/information that should be explored.
            actionItemList = {}
            currentClusterKey = "cluster(%s)" %(cce.getClusterName())
            actionItemList[currentClusterKey] = []
            
            result = cce.isClusterConfFilesIdentical(self.__cnc.getPathToClusterConfFiles())
            self.writeTestResult(filename, "Testing if the cluster nodes have identical cluster.conf files.", result)
            result = (int(cce.getPostJoinDelay()) > 3)
            self.writeTestResult(filename, "Testing to make sure post_join_delay is greater than 3 seconds in cluster.conf.", result)
            result = (int(cce.getPostFailDelay()) == 0)
            self.writeTestResult(filename, "Testing to make sure post_fail_delay is 0 in cluster.conf.", result)
            result = cce.isFencingEnabledOnAllNodes()
            self.writeTestResult(filename, "Testing if a fence agent is defined on all cluster nodes.", result)
            self.write(filename, "")

            # ###################################################################
            # Verify the cluster node configuration
            # ###################################################################
            self.writeSeperator(filename, "Cluster Node Configuration Summary");
            for clusternode in self.__cnc.getClusterNodes():
                currentNodeKey = "Node(%s)" %(clusternode.getHostname())
                actionItemList[currentNodeKey] = []

                self.write(filename, "%s:" %(clusternode.getHostname()))
                cce = ClusterConfEvaluator(self.__cnc.getBaseClusterConfFile())
                clusternodeNames = cce.getClusterNodeNames()

                # Node Names Testing
                result = self.__isClusterNodeNamesInHostsFile(clusternodeNames, clusternode.getNetworkMaps())
                self.writeTestResult(filename,"Testing if all cluster node names are defined in /etc/hosts.", result)

                result = self.__isClusterNodeNamesRFC2181Valid(clusternodeNames)
                self.writeTestResult(filename, "Testing if all cluster node names are RFC2181 compliant.", result)
                                     
                # Fence Testing
                result = clusternode.isManualFencingEnabled()
                self.writeTestResult(filename, "Testing if fence_manual is defined as fence agent for the node.", not result)
                if (result) :
                    actionItemList.get(currentNodeKey).append("Manual fencing is unsupported, verify that other fence devices exist.")
                    
                # Check if acpi is disabled if sys mgmt card is fence device
                if (cce.isSystemManagementFencingEnabled()):
                    resultRL = clusternode.isAcpiDisabledinRunlevel()
                    self.writeTestResult(filename, "Verifing that the service \"acpid\" is disabled for runlevels 3-5 since system management fence device detected.", resultRL)
                    resultGrub = clusternode.isAcpiDisabledinCmdLine()
                    self.writeTestResult(filename, "Verfiying that \"acpi\" is disabled in kernel via /proc/cmdline since system management fence device detected.", resultGrub)

                # Check if services are enabled
                self.__verifyServicesStatus(filename, clusternode.getClusterServicesStatus())

                # Check if GFS2 module should be removed
                result = self.__doesGFS2ModuleRemoved(clusternode.getUnameA(), clusternode.getClusterModulePackagesVersion())
                self.writeTestResult(filename, "Testing if kmod-gfs2 is not installed on a running kernel >= 2.6.18-128.", not result)
                
                # Check for multicast tags on RH5 nodes 
                distroRelease = clusternode.getDistroRelease() 
                if ((distroRelease.getDistroName() == "RHEL") and (distroRelease.getMajorVersion() >= 5)):
                    cnp = clusternode.getClusterNodeProperties()
                    result = ((len(cnp.getMulticastAddress()) > 0) or (len(cnp.getMulticastInterface()) > 0))
                    self.writeTestResult(filename, "Verifying that multicast tags are not in <clusternodes> stanzas.", not result)

                # Add newline to separate the node stanzas
                self.write(filename, "")
            # ###################################################################
            # These are items that should be explored as possible
            # problems with cluster. Still working on this and how to
            # approach or even if I will add action item stuff.
            #
            # Well explore in next version this is just for devel ref.
            # ###################################################################
            # For example add an item after test failed:
            # actionItemList.get(currentNodeKey).append("RHEL 5
            # clustering ignores multicast tags on nodes. These should
            # be removed.")

            # Then cycle through and  add them
            # headerWrote = False
            # keys = actionItemList.keys()
            # keys.sort()
            # for key in keys:
            #     for message in actionItemList.get(key):
            #         if ((not headerWrote) and (len(message))):
            #             self.writeSeperator(filename, "Action Items")
            #             headerWrote = True
            #         self.write(filename, "%s: %s" %(key, message))
                           
