#!/usr/bin/env python
"""
This class does various operations on a /etc/cluster/cluster.conf file
that is in xml format.

@author    :  Shane Bradley
@contact   :  sbradley@redhat.com
@version   :  2.03
@copyright :  GPLv2
"""
import os.path
import re
import libxml2
import string
import logging
import fileinput

import sx
import sx.tools

class ClusterNodeProperties:
    def __init__(self, nodeName, nodeID, votes, multicastAddress, multicastInterface, 
                 cmanMulticastAddress, fenceDevices) :
        """
        @param nodeName: The name of node.
        @type nodeName: String
        @param nodeID: The id of the node.
        @type nodeID: String
        @param votes: The votes for the node.
        @type votes: String
        @param multicastAddress: The multicast address of the node.
        @type multicastAddress: String
        @param multicastInterface: The multicast interface for the node.
        @type multicastInterface: String
        @param cmanMulticastAddress: The cman multicast address for the node.
        @type cmanMulticastAddress: String
        @param fenceDevices: A dictionary of fence devices.
        @type fenceDevices: Array
        """
        self.__nodeName = nodeName
        self.__nodeID = nodeID
        self.__votes = votes
        self.__multicastAddress = multicastAddress
        self.__multicastInterface = multicastInterface
        self.__cmanMulticastAddress = cmanMulticastAddress
        # List of dictionaries
        self.__fenceDevices = fenceDevices

    def getNodeName(self):
        """
        Returns the node name.

        @return: Returns the node name.
        @rtype: String
        """
        if (self.isEmpty()):
            return ""
        return self.__nodeName

    def getNodeID(self) :
        """
        Returns the nodeID.

        @return: Returns the nodeID.
        @rtype: String
        """
        if (self.isEmpty()):
            return ""
        return self.__nodeID

    def getVotes(self) :
        """
        Returns the votes for the node.

        @return: Returns the votes for the node.
        @rtype: String
        """
        if (self.isEmpty()):
            return ""
        return self.__votes

    def getMulticastAddress(self):
        """
        Returns the multicast address.

        @return: Returns the multicast address.
        @rtype: String
        """
        if (self.isEmpty()):
            return ""
        return self.__multicastAddress

    def getMulticastInterface(self):
        """
        Returns the multicast interface.

        @return: Returns the multicast interface.
        @rtype: String
        """
        if (self.isEmpty()):
            return ""
        return self.__multicastInterface

    def getCmanMulticastAddress(self):
        """
        Returns the cman multicast address.

        @return: Returns the cman multicast address.
        @rtype: String
        """
        if (self.isEmpty()):
            return ""
        return self.__cmanMulticastAddress

    def getFenceDevices(self) :
        """
        Returns the node name.

        @return: Returns the fence devices for the node.
        @rtype: Dictionary
        """
        if (self.isEmpty()):
            return {} 
        return self.__fenceDevices

    def isEmpty(self) :
        """
        If the nodeName is an empty string this object is considered
        empty. Returns True if nodeName is not empty.

        @return: Returns True if nodeName is not empty.
        @rtype: Boolean
        """
        # If node name is empty then considered data empty because data not associated with node name
        return (not len(self.__nodeName)> 0)

class ClusterConfEvaluator :
    """
    This class does various operations on a cluster.conf file that is
    in xml format.
    """
    def __init__(self, pathToClusterConf) :
        """
        Setups the cluster xml xpathcontext for the file. It will add
        in "" for password section if needed.

        @param pathToClusterConf: Path to the cluster.conf file that is an
        xml file.
        @type pathToClusterConf: String
        """
        # Ignore parser errors on cluster.conf files.
        #import warnings
        #warnings.filterwarnings("ignore", ": parser erwarror : ")

        self.__pathToClusterConf = pathToClusterConf
        # Fence device password should have quotes
        if (os.path.exists(self.__pathToClusterConf)) :
            # foundXMLHeader = False
            for line in fileinput.FileInput(self.__pathToClusterConf,inplace=1):
                if (line.find("=***") >= 0):
                    line = re.sub("=\*\*\*", "=\"***\"", line)
                print line,
        else:
            message = "The cluster.conf file does not exist: %s" %(self.__pathToClusterConf)
            logging.getLogger(sx.MAIN_LOGGER_NAME).error(message)
            self.__isXMLValid = False
            return
        # Setup XML xpath context
        self.__xpathContext = None
        self.__isXMLValid = True
        try: 
            xml = libxml2.parseFile(self.__pathToClusterConf)
            self.__xpathContext = xml.xpathNewContext()
        except libxml2.parserError:
            message = "Parsing error occured reading the cluster.conf file." 
            logging.getLogger(sx.MAIN_LOGGER_NAME).error(message)
            self.__isXMLValid = False

    # #######################################################################
    # Is Functions
    # #######################################################################
    def isClusterConfFilesIdentical(self,listOfFiles) :
        if (not len(listOfFiles) > 1):
            return False
        return  sx.tools.isFilesIdentical(self.__pathToClusterConf, listOfFiles)

    def isXMLValid(self) :
        """
        If there is parsing errors opening xml file up then file is
        invalid.

        @return: Returns False if the file contains parsing errors,
        otherwise returns True.
        @rtype: Boolean
        """
        return self.__isXMLValid

    def isFencingEnabledOnAllNodes(self) :
        """
        Returns True if all clusternodes have fencing agent defined.
        
        @return: Trie of all clusternodes have fencing agent defined.
        @rtype: Boolean
        """
        if len(self.__xpathContext.xpathEval("/cluster/clusternodes/clusternode[not(fence/method/device)]")):
            return  False
        return True

    def isManualFencingEnabledOnAnyNode(self):
        """
        Returns True if they have the agent fence_manual enabled on
        any clusternode.

        @return: Returns True if they have the agent fence_manual
        enabled on any clusternode.
        @rtype: Boolean
        """
        if len(self.__xpathContext.xpathEval(
                "/cluster/clusternodes/clusternode[/cluster/fencedevices/fencedevice[@agent='fence_manual']/@name=fence/method/device/@name]")):
            return True
        return False

    def isSystemManagementFencingEnabled(self):
        """
        Returns True if any of the fence agents defined on a
        clusternode is a systems management card: ilo, rsa, drac.

        @return: Returns True if any of the fence agents defined on a
        clusternode is a systems management card: ilo, rsa, drac.
        @rtype: Boolean
        """
        if len(self.__xpathContext.xpathEval(
                "/cluster/clusternodes/clusternode[/cluster/fencedevices/fencedevice[@agent='fence_ilo']/@name=fence/method/device/@name]")):
            return True
        elif len(self.__xpathContext.xpathEval(
                "/cluster/clusternodes/clusternode[/cluster/fencedevices/fencedevice[@agent='fence_rsa']/@name=fence/method/device/@name]")):
            return True
        elif len(self.__xpathContext.xpathEval(
                "/cluster/clusternodes/clusternode[/cluster/fencedevices/fencedevice[@agent='fence_drac']/@name=fence/method/device/@name]")):
            return True
        return False 

    def isQDiskEnabledWithHeurtistics(self):
        """
        Returns True if their quorumd is enable in their cluster.conf.
 
        @return: Returns True if their quorumd is enable in their
        cluster.conf.
        @rtype: Boolean
        """
        if (len(self.__xpathContext.xpathEval('/cluster/quorumd/heuristic')) > 0):
            return True
        return False

    # #######################################################################
    # Get Functions
    # #######################################################################
    def getClusterName(self) :
        """
        Returns the name of the cluster.

        @return: Returns the name of the cluster.
        @rtype: String
        """
        if (len(self.__xpathContext.xpathEval('/cluster/@name')) > 0):
            return self.__xpathContext.xpathEval('/cluster/@name').pop().content
        return ""

    def getClusterNodeNames(self) :
        """
        Returns an array of strings for all the node names in
        cluster.conf.
        
        @return: Returns an array of strings for all the node names in
        cluster.conf.
        @rtype: Array
        """
        hostnames = []
        for hostname in self.__xpathContext.xpathEval('/cluster/clusternodes/clusternode/@name'):
            hostnames.append(hostname.content) 
        return hostnames

    def getPostFailDelay(self) :
        """
        Returns the post_fail_delay value in fence_daemon properties.

        @return: The value of string post_join_delay in xml.
        @rtype: String
        """
        if (len(self.__xpathContext.xpathEval('/cluster/fence_daemon/@post_fail_delay')) > 0):
            return self.__xpathContext.xpathEval('/cluster/fence_daemon/@post_fail_delay').pop().content
        return "0"

    def getPostJoinDelay(self) :
        """
        Returns the post_join_delay value in fence_daemon properties.

        @return: The value of string post_join_delay in xml.
        @rtype: String
        """
        if (len(self.__xpathContext.xpathEval('/cluster/fence_daemon/@post_join_delay')) > 0):
            return  self.__xpathContext.xpathEval('/cluster/fence_daemon/@post_join_delay').pop().content
        return "3"

    def getCmanMulticastAddress(self) :
        """
        Returns the post_join_delay value in fence_daemon properties.

        @return: The value of string post_join_delay in xml.
        @rtype: String
        """
        if (len(self.__xpathContext.xpathEval('/cluster/cman/multicast/@addr')) > 0):
            return  self.__xpathContext.xpathEval('/cluster/cman/multicast/@addr').pop().content
        return ""

    def getFenceDeviceProperties(self) :
        """
        This function will return an array of dictionaries for each
        fencedevice list.

        @return: Returns an array of dictionaries for each fencedevice
        list.
        @rtype: Array
        """
        fenceDeviceList = [] 
        for fenceDeviceElement in self.__xpathContext.xpathEval('/cluster/fencedevices/fencedevice'):
            dict = {} 
            try:
                dict["name"] = fenceDeviceElement.xpathEval("@name")[0].content
                dict["agent"] = fenceDeviceElement.xpathEval("@agent")[0].content 
            except IndexError:
                continue        
            try: 
                dict["hostname"] = fenceDeviceElement.xpathEval("@hostname")[0].content
            except IndexError:
                try:
                    dict["ipaddr"] = fenceDeviceElement.xpathEval("@ipaddr")[0].content
                except IndexError:
                    pass
            try: 
                dict["login"] = fenceDeviceElement.xpathEval("@login")[0].content
            except IndexError:
                pass
            fenceDeviceList.append(dict)
        return fenceDeviceList

    def __getFenceDeviceProperty(self, fenceDeviceName) :
        fenceDeviceList = self.getFenceDeviceProperties() 
        for fDeviceProperty in fenceDeviceList:
            if (fDeviceProperty.has_key("name")) :
                if (fDeviceProperty.get("name") == fenceDeviceName):
                    return fDeviceProperty
        return {}

    def getClusterNodeProperties(self, clusterNodeName) :
        """
        This function will return a dictionary of the properties in
        the clusternode section.
        
        @return: Returns a dictionary of the properties in the
        cluster.conf.
        @rtype: Dictionary
        """
        cnpMap = {} 
        nodeName = ""
        nodeID = ""
        votes = ""
        multicastAddress = ""
        multicastInterface = ""
        fenceDevices = {}
        cmanMulticastAddress = self.getCmanMulticastAddress() 
        if (not len(clusterNodeName) > 0):
            # Just return empty property
            return ClusterNodeProperties(nodeName, nodeID, votes, multicastAddress, multicastInterface, cmanMulticastAddress, fenceDevices) 
        try :
            clusterNodeElement = self.__xpathContext.xpathEval("/cluster/clusternodes/clusternode[@name='%s']" % clusterNodeName)[0]
            nodeName = clusterNodeElement.xpathEval("@name")[0].content
            nodeID = clusterNodeElement.xpathEval("@nodeid")[0].content
            votes = clusterNodeElement.xpathEval("@votes")[0].content
        except IndexError:
            pass
        try :
            clusterNodeMulticastElement = clusterNodeElement.xpathEval("multicast")[0]
            multicastAddress = clusterNodeMulticastElement.xpathEval("@addr")[0].content
            multicastInterface = clusterNodeMulticastElement.xpathEval("@interface")[0].content
        except IndexError:
            pass
        try:
            clusterNodeFenceElements = clusterNodeElement.xpathEval("fence/method")
            clusterNodeFenceDict = {} 
            for fenceMethodElement in clusterNodeFenceElements:
                methodName = fenceMethodElement.xpathEval("@name")[0].content
                clusterNodeFenceDict[methodName] = []
                deviceElements = fenceMethodElement.xpathEval("device")
                for deviceElement in deviceElements:
                    fenceDeviceName = deviceElement.xpathEval("@name")[0].content
                    clusterNodeFenceDict.get(methodName).append(self.__getFenceDeviceProperty(fenceDeviceName))
            fenceDevices = clusterNodeFenceDict
        except IndexError:
            pass
        return ClusterNodeProperties(nodeName, nodeID, votes, 
                                     multicastAddress, multicastInterface, 
                                     cmanMulticastAddress, fenceDevices) 
