#!/usr/bin/env python
"""
This classes are for cluster nodes in the sosreport/sysreport. They
represnet configs files paths and other items that are needed for all
cluster tools.

@author    :  Shane Bradley
@contact   :  sbradley@redhat.com
@version   :  2.03
@copyright :  GPLv2
"""
import re
import os.path
import logging

import sx
from sx.logwriter import LogWriter
from sx.plugins.lib.clusterconfevaluator import ClusterConfEvaluator
from sx.plugins.lib.clusterconfevaluator import ClusterNodeProperties
from sx.reports.sosreport import NetworkMap

class ClusterNodeNetworkMap(NetworkMap):
    """
    Container for network information for network information.
    """
    def __init__(self, interface, hwAddr,ipAddr,
                 subnetMask, etcHostsMap, clusterNodeName):
        """
        @param interface: The network interface.
        @type interface: String
        @param hwAddr: The hardware address
        @type hwAddr: String
        @param ipAddr: The ip address.
        @type ipAddr: String
        @param subnetMask: The subnet mask.
        @type subnetMask: String
        @param etcHostsMap: This is a dictionary of all ip/hostname
        entries in /etc/hosts file.
        @type etcHostsMap: Dictionary
        @param clusterNodeName : The hostname of the clusternode
        used in cluster.conf if it exists.
        @type clusterNodeName: String
        """
        NetworkMap.__init__(self, interface, hwAddr, ipAddr, subnetMask, etcHostsMap)
        self.__clusterNodeName = clusterNodeName

    def __str__(self) :
        """
        Returns a string that is composed of the name and description.
        
        @return: Returns a string that is composed of the name and
        description.
        @rtype: String
        """
        returnString = ""
        returnString += "interface:   %s\n" % (self.getInterface())
        returnString += "ip address:  %s\n" % (self.getIPAddress())
        returnString += "hw address:  %s\n" % (self.getHardWareAddress())
        returnString += "hostnames:   " 
        for hostname in self.getHostnames() :
            returnString += "%s " %(hostname)
        returnString += "\n"
        returnString += "node name:   %s\n" % (self.getClusterNodeName())
        return returnString

    def getClusterNodeName(self) :
        """
        If empty string is returned hen no clusternode name is associated with
        this interface/ip address. However, if not empty string then
        this is the NetworkMap that clusternode uses for
        communication.

        @return: If empty string is returned hen no clusternode name
        is associated with this interface/ip address. However, if not
        empty string then this is the NetworkMap that clusternode uses
        for communication.
        @rtype: String
        """
        return self.__clusterNodeName

# ###############################################################################
# ClusterNode
# ###############################################################################
class ClusterNode:
    """
    This classes are for cluster nodes in the
    sosreport/sysreport. They represnet configs files paths and other
    items that are needed for all cluster tools.
    """
    def __init__(self, pathToClusterConf, distroRelease, uname_a, hostname,
                 uptime, networkMaps, clusterServicesStatus, pathToClusterCommands, 
                 clusterPackageVersions, clusterModulePackageVersions, 
                 isAcpiDisabledInCmdLine, isAcpiDisabledInRunLevel) :
        """
        Requries the cluster.conf(and file has to exist) so we know
        that this node is apart of cluster. The pathToClusterConf
        should not be equal to None. The path must exist.

        If cluster.conf does not exist then this class should not be
        created.

        @param pathToClusterConf: This is the path to the
        /etc/cluster/cluster.conf file.
        @type pathToClusterConf: String
        @param distroRelease: A DistroRelease object that describes
        the type of Operating System Distrobution this cluster.conf is
        from.
        @type distroRelease: DistroRelease
        @param uname_a: The uname -a data for the node
        @type uname_a: String
        @param hostname: The hostname of the node.
        @type hostname: String
        @param uptime: The uptime data for the node.
        @type uptime: String
        @param networkMaps: Array of NetworkMap objects
        @type networkMaps: Array
        @param clusterServicesStatus: List of cluster services runlevel 
        status on enabled/disabled in runleve 3-5.
        @type clusterServicesStatus: Array
        @param pathToClusterCommands: Path to the cluster commands.
        @type pathToClusterCommands: String
        @param clusterPackageVersions: List of installed cluster packages
        @type clusterPackageVersions: Array
        @param clusterModulePackageVersions: List of installed cluster module packages.
        @type clusterModulePackageVersions: Array
        @param isAcpiDisabledInCmdLine: True if acpi is disabled in grub.
        @type isAcpiDisabledInCmdLine: Boolean
        @param isAcpiDisabledInRunLevel: True if acpi is disabled in runlevels.
        @type isAcpiDisabledInRunLevel: Boolean

        """
        self.__pathToClusterConf = pathToClusterConf
        self.__distroRelease = distroRelease

        self.__uname_a = uname_a
        self.__hostname = hostname
        self.__clusterCommandsDict = self.__mapPathsToClusterCommands(pathToClusterCommands)
        self.__isAcpiDisabledInCmdLine = isAcpiDisabledInCmdLine
        self.__isAcpiDisabledInRunLevel = isAcpiDisabledInRunLevel
        self.__clusterPackageVersions = clusterPackageVersions
        self.__clusterModulePackageVersions = clusterModulePackageVersions
        self.__clusterServicesStatus = clusterServicesStatus
        self.__uptime = uptime

        # network maps of the clusternode, set to None so if None it
        # generates the mapping. There is no corresponding set method
        # and will be set on first corresponding call to get.
        self.__networkMaps = self.__generateClusterNodeNetworkMaps(networkMaps, self.__pathToClusterConf)

        # this is properties of the node in the cluster.conf section
        # <clusternode>, by default it is Node and will need to be set
        # on first call to get.
        clusterNodeName = ""
        cnmap = self.getClusterNodeHeartbeatNetworkMap()
        if (not cnmap == None):
            clusterNodeName = cnmap.getClusterNodeName()
        cce = ClusterConfEvaluator(self.__pathToClusterConf)
        self.__clusterNodeProperties = cce.getClusterNodeProperties(clusterNodeName)


    def __str__(self) :
        """
        Returns a string that is composed of the name and description.
        
        @return: Returns a string that is composed of the name and
        description.
        @rtype: String
        """
        cnp = self.getClusterNodeProperties()
        if (cnp.isEmpty()):
            return ""
        nameMessage = "Node Name:    %s\n" %(cnp.getNodeName())

        interFaceMessage = "Heartbeat Interface: %s IPAddress: %s \n" % ("    ", "    ")
        cnmap = self.getClusterNodeHeartbeatNetworkMap()
        if (not cnmap == None) :
            ipAddr = cnmap.getIPAddress()
            interface = cnmap.getInterface()
            if ((len(ipAddr) > 0) and (len(interface) > 0)):
                interFaceMessage = "HB Interface: %s(%s) \n" % (ipAddr, interface)
        
        multicastMessage = ""
        cmanMulticastAddress = cnp.getCmanMulticastAddress()
        if (len(cmanMulticastAddress) > 0):
            multicastMessage += "CMAN MC Addr: %s \n" %(cmanMulticastAddress)
        multicastAddress = cnp.getMulticastAddress()
        multicastInterface = cnp.getMulticastInterface()
        if ((len(multicastAddress) > 0) and (len(multicastInterface) > 0)):
            multicastMessage += "MC Addr:      %s(%s) \n" %(multicastAddress, multicastInterface)
    
        fenceMessage = ""
        fenceDevicesDict = cnp.getFenceDevices()
        if (len(fenceDevicesDict) > 0):
            fenceDevicesKeys = fenceDevicesDict.keys()
            fenceDevicesKeys.sort()
            for key in fenceDevicesKeys:
                fenceDevicesMessage = ""
                # for devices on same fence level
                for device in fenceDevicesDict.get(key):
                    if device.has_key("name") :
                        if (not len(fenceMessage) > 0) :
                            fenceMessage += "Fence Dev:    %s(Level %s)\n" %(device.get("name"), key)
                        else:
                            fenceMessage += "              %s(Level %s)\n" %(device.get("name"), key)
        return  "%s%s%s%s" %(nameMessage, interFaceMessage, multicastMessage, fenceMessage)


    # #######################################################################
    # Private helper methods for get functions
    # #######################################################################
    def __generateClusterNodeNetworkMaps(self, networkMaps, pathToClusterConf) :
        """
        The function returns a ClusterNodeNetworkMap based on
        NetworkMaps. It uses cluster.conf to get multicast and node
        name information.

        @return: Returns a list of ClusterNodeNetworkMaps.
        @rtype: Array
        
        @param networkMaps: A list of NetworkMap objects for the
        cluster node.
        @type networkMaps: Array
        @param pathToClusterConf: The path to the cluster.conf.
        @type pathToClusterConf: String
        """
        cce = ClusterConfEvaluator(pathToClusterConf)
        nodeNames = cce.getClusterNodeNames()
        clusterNodeNetworkMaps = []
        for nMap in networkMaps:
            # If this is heartbeat nic for cluster
            # communication set the clusterNodeName to
            # hostname that matches clusternode name
            clusterNodeName = ""
            hostnames = nMap.getHostnames() 
            for nodeName in nodeNames:
                if (nodeName in hostnames) :
                    clusterNodeName = nodeName
                    break;
            # Append map to list of maps
            clusterNodeNetworkMaps.append(ClusterNodeNetworkMap(nMap.getInterface(),
                                                                nMap.getHardWareAddress(),
                                                                nMap.getIPAddress(),
                                                                nMap.getSubnetMask(),
                                                                nMap.getEtcHostsMap(),
                                                                clusterNodeName))
        return clusterNodeNetworkMaps

    def __mapPathsToClusterCommands(self, pathToClusterCommands) :
        """
        This function sets the path to the directory that contain all
        the cluster command files that were generated. This is the
        directory path and not an actual file path.

        A dictionary will be created to acces the files within that
        directory.

        The current keys are:
        "ccs_tool_lsnode", "cman_tool_-af_nodes", "cman_tool_status",
        "ipvsadm_-L", "rg_test_test_.etc.cluster.cluster.conf",
        "clustat", "cman_tool_services", "fdisk_-l",
        "openais-cfgtool_-s"]


        @param pathToClusterCommands: This function sets the path to
        the directory that contain all the cluster command files that
        were generated. This is the directory path and not an actual
        file path.
        @type String
        """
        if (not len(pathToClusterCommands) > 0) :
            return
        if os.path.isfile(pathToClusterCommands) :
            clusterCommandsFilenames = ["ccs_tool_lsnode", "cman_tool_-af_nodes",
                                        "cman_tool_status",  "ipvsadm_-L",
                                        "rg_test_test_.etc.cluster.cluster.conf",
                                        "clustat",
                                        "cman_tool_services",
                                        "fdisk_-l",
                                        "openais-cfgtool_-s"]
            for filename in clusterCommandsFilenames:
                pathToFilename = os.path.join(pathToClusterCommands, filename)
                if os.path.isfile(pathToFilename) :
                    self.__clusterCommandsDict[filename] = pathToFilename

    
    # #######################################################################
    # Is methods
    # #######################################################################
    def isAcpiDisabledinCmdLine(self) :
        """
        Returns True if acpi is disabled in /proc/cmdline

        @return: True if acpi is disabled in /proc/cmdline
        @rtype: Boolean
        """
        return self.__isAcpiDisabledInCmdLine

    def isAcpiDisabledinRunlevel(self) :
        """
        Returns True if acpi is disabled in runleve

        @return: True if acpi is disabled in runlevel.
        @rtype: Boolean
        """
        return self.__isAcpiDisabledInRunLevel

    def isManualFencingEnabled(self) :
        cnp = self.getClusterNodeProperties()
        fenceDevicesDict = cnp.getFenceDevices()
        for key in fenceDevicesDict.keys():
            for fenceDevice in fenceDevicesDict.get(key):
                fenceAgent = fenceDevice.get("agent")
                if(fenceAgent == "fence_manual"):
                    return True
        return False

    # #######################################################################
    # Get methods
    # #######################################################################
    def getDistroRelease(self) :
        """
        Returns the DistroRelease Object for this node.

        @return: Returns the DistroRelease Object for this node.
        @rtype: DistroRelease
        """
        return self.__distroRelease

    def getPathToClusterConf(self) :
        """
        Returns the path to the cluster.conf file.

        @return: Returns the path to the cluster.conf.
        file.
        @rtype: String
        """
        return self.__pathToClusterConf

    def getUnameA(self) :
        """
        Returns the data from the uname_a file.

        @return: Returns the data from the uname_a file.
        @rtype: String
        """
        return self.__uname_a
    def getHostname(self) :
        """
        Returns the hostname.

        @return: Returns the hostname.
        @rtype: String
        """
        return self.__hostname

    def getPathToClusterCommand(self, key) :
        """
        Returns a path to a cluster command file based on the key.

        The current keys are:
        "ccs_tool_lsnode", "cman_tool_-af_nodes", "cman_tool_status",
        "ipvsadm_-L", "rg_test_test_.etc.cluster.cluster.conf",
        "clustat", "cman_tool_services", "fdisk_-l",
        "openais-cfgtool_-s"]

        @return: Returns a path to a cluster command file.
        @rtype: String

        @param key: This is file that will be returned.
        @type key: String
        """
        return self.__clusterCommandsDict.get(key)

    def getClusterCommandsKeys(self) :
        """
        Returns an array of keys for cluster commmands.

        The current keys are:
        "ccs_tool_lsnode", "cman_tool_-af_nodes", "cman_tool_status",
        "ipvsadm_-L", "rg_test_test_.etc.cluster.cluster.conf",
        "clustat", "cman_tool_services", "fdisk_-l",
        "openais-cfgtool_-s"]

        @return: Returns an array of keys.
        @rtype: Array
        """
        return self.__clusterCommandsDict.keys()


    def getClusterPackagesVersion(self) :
        """
        Returns an array of installed cluster packages.

        @return: Returns an array of installed cluster
        packages.
        @rtype: Array
        """
        return self.__clusterPackageVersions

    def getClusterModulePackagesVersion(self) :
        """
        Returns an array of installed cluster module packages.

        @return: Returns an array of installed cluster module
        packages.
        @rtype: Array
        """
        return self.__clusterModulePackageVersions

    def getClusterServicesStatus(self) :
        """
        Returns an array of dictionaries of dictionaries.
        Example:
        [{'cman': {3: False, 4: False, 5: False}}, {'clvmd': {}}, {'gfs': {}}, {'rgmanager': {3: False, 4: False, 5: False}}]

        @return: Returns an array of dictionaries of dictionaries for
        service status.
        @rtype: Array
        """
        return self.__clusterServicesStatus

    def getUptime(self) :
        """
        Returns the uptime for the cluster node.

        @return: Returns the uptime for the cluster node.
        @rtype: String
        """
        return self.__uptime

    def getNetworkMaps(self) :
        """
        Returns all the generated NetworkMaps Objects for this
        clusternode.

        @return: Returns all the generated NetworkMaps Objects for
        this clusternode.
        @rtype: Array
        """
        return self.__networkMaps

    def getClusterNodeHeartbeatNetworkMap(self) :
        """
        Returns the NetworkMap object associated with heartbeat or
        cluster communication network. If there is not a mapping of
        the cluster node name to nic device via hosts file then None
        is returned.

        @return: Returns the NetworkMap object associated with
        heartbeat or cluster communication network.
        @rtype: NetworkMap
        """
        for cnmap in self.getNetworkMaps() :
            if (len(cnmap.getClusterNodeName()) > 0) :
                return cnmap
        return None

    def getClusterNodeProperties(self) :
        """
        Returns the clusternode properties from the cluster.conf as
        dictionary.

        @return: Returns the clusternode properties from the
        cluster.conf as dictionary.
        @rtype: Dictionary
        """
        return self.__clusterNodeProperties

