#!/usr/bin/env python
"""
This file contains a container class for a collection of clusternode
objects.

@author    :  Shane Bradley
@contact   :  sbradley@redhat.com
@version   :  2.03
@copyright :  GPLv2
"""
import re
import os.path
import logging

import sx
from sx.logwriter import LogWriter
from sx.plugins.lib.clusterconfevaluator import ClusterConfEvaluator
from sx.plugins.lib.clusternode import ClusterNode
from sx.plugins.lib.clusternode import ClusterNodeNetworkMap

class ClusterNodes:
    
    PACKAGE_LIST = ["ccs", "cman", "fenced", "magma", "magma-plugins", "rgmanager",
                    "dlm", "GFS", "gfs-utils", "gfs2-utils", "lvm2-cluster", "cmirror",
                    "system-config-cluster", "luci", "ricci", "modcluster",
                    "cluster-snmp", "cluster-cim", "openais"]

    MODULE_LIST =  ["kmod-gfs2", "kmod-gfs2-xen", "kmod-gfs", "kmod-gfs-xen""cmirror", "cmirror-xen",
                    "cman-kernel", "cman-kernel-smp", "cman-kernel-largesmp", "cman-kernel-xenU", "cman-kernel-hugemem",
                    "dlm-kernel", "dlm-kernel-smp", "dlm-kernel-largesmp", "dlm-kernel-xenU", "dlm-kernel-hugemem",
                    "cmirror-kernel", "cmirror-kernel-smp", "cmirror-kernel-largesmp", "cmirror-kernel-xenU", "cmirror-kernel-hugemem",
                    "GFS-kernel", "GFS-kernel-smp", "GFS-kernel-largesmp", "GFS-kernel-xenU", "GFS-kernel-hugemem"]

    RHEL4_CLUSTER_SERVICES = { 1:"ccsd", 2:"cman", 3:"qdiskd", 4:"fenced", 5:"clvmd", 6:"gfs", 7:"rgmanager" }
    RHEL5_CLUSTER_SERVICES = { 1:"cman", 2:"qdiskd", 3:"clvmd", 4:"gfs", 5:"rgmanager" }

    def __init__(self) :
        self.__clusterNodes = []

    def count(self):
        """
        Returns the number of nodes.

        @return: Returns the number of nodes that are in the array.
        @rtype: Int
        """
        return len(self.__clusterNodes)

    def getClusterName(self):
        """
        Returns the name of the cluster.

        @return: Returns the name of the cluster.
        @rtype: String
        """
        if (len(self.__clusterNodes) > 0):
            cce = ClusterConfEvaluator(self.getBaseClusterConfFile())
            return cce.getClusterName()
        return ""

    def getClusterNodes(self) :
        """
        Returns an array of ClusterNode objects.

        @return: Returns an array of ClusterNode objects.
        @rtype: Array
        """
        # Sort by hostname because array order is preserved
        hostnames = []
        for node in self.__clusterNodes:
            hostnames.append(node.getHostname())
        hostnames.sort()

        # Create the new array to return
        sortedClusterNodes = []
        for hostname in hostnames:
            for node in self.__clusterNodes:
                if (hostname == node.getHostname()):
                    sortedClusterNodes.append(node)
        return sortedClusterNodes

    def getClusterNodeNames(self) :
        """
        Returns an array of cluster node names. If name is not found
        then the following is used for the name:
        "%s-unknown_node_name" %(node.getHostname().
        
        @return: Returns a list of cluster node names.
        @rtype: Array
        """
        nodeNameList = []
        nodes = self.getClusterNodes()
        for node in nodes:
            nodeName = "%s-unknown_node_name" %(node.getHostname())
            cnmap = node.getClusterNodeHeartbeatNetworkMap()
            if (not cnmap == None):
                nodeName = cnmap.getClusterNodeName()
            nodeNameList.append(nodeName)
        return nodeNameList

    def getBaseClusterConfFile(self) :
        """
        Returns the path to the base cluster.conf file that all
        comparsion and checks are filled against.

        @return: Returns the path to the base cluster.conf file that
        all comparsion and checks are filled against.  
        @rtype: String
        """
        if (len(self.__clusterNodes) > 0):
            return self.__clusterNodes[0].getPathToClusterConf()
        return ""

    def getPathToClusterConfFiles(self) :
        """
        Returns an array of all cluster.conf paths.

        @return: Returns an array of all cluster.conf paths.
        @rtype: String
        """
        pathToClusterConfFilesList = []
        for clusterNode in self.getClusterNodes():
            pathToFile = clusterNode.getPathToClusterConf()
            if (len(pathToFile) > 0) :
                pathToClusterConfFilesList.append(pathToFile)
        return pathToClusterConfFilesList

    def listClusterNodesMissingReports(self):
        """
        Returns a list of clusternodes that did not have a report
        extracted.
        
        @return: Returns a list of clusternodes that did not have a
        report extracted.
        @rtype: Array
        """
        # Nodes that are in cluster.conf, so should have report of all these
        cceList = ClusterConfEvaluator(self.getBaseClusterConfFile()).getClusterNodeNames()
        # Nodes that are in this collection that actual did have correct report
        cncList = self.getClusterNodeNames()
        return list(set(cceList) - set(cncList))

    def add(self, report) :
        """
        This function will add only valid reports from clusternodes to
        this object.

        @param report: A SReport object that will be added to list of
        clusternodes if it is valid clusternode.
        @type report: SReport
        """
        # Verify that cluster.conf exists and plugin will work
        # with the distro release
        pathToClusterConfFile = report.extractPath("etc/cluster/cluster.conf")
        distroRelease = report.getRedhatReleaseVersion()
        # cce will verify that cluster.conf is valid xml
        cce = ClusterConfEvaluator(pathToClusterConfFile)
        # ###############################################################
        # If distro release is not supported or cluster.conf
        # does not validate to be true then the node will not
        # be added.
        # ###############################################################
        if (distroRelease == None) :
            message = "The disto release file was either not valid or unknown type."
            logging.getLogger(sx.MAIN_LOGGER_NAME).error(message)
            return False
        elif (not ((distroRelease.getDistroName() == "RHEL") and
                   ((distroRelease.getMajorVersion() == 5) or 
                    (distroRelease.getMajorVersion() == 4)))):
            message = "The disto release is not supported."
            logging.getLogger(sx.MAIN_LOGGER_NAME).error(message)
            return False
        elif ((not len(pathToClusterConfFile) > 0) or (not os.path.exists(pathToClusterConfFile))) :
            message = "The path to the cluster.conf does not exists: %s" %(pathToClusterConfFile)
            logging.getLogger(sx.MAIN_LOGGER_NAME).error(message)
            return False
        elif (not cce.isXMLValid()):
            message = "The cluster.conf file is not a valid xml file: %s" %(report.getHostname())
            logging.getLogger(sx.MAIN_LOGGER_NAME).error(message)
            return False

        # ###############################################################
        # Create the node since it is valid then append to collection
        # ###############################################################
        pathToClusterCommands = report.extractPath("sos_commands/cluster")
        if (not len(pathToClusterCommands) > 0) :
            pathToClusterCommands = ""

        # Find out if acpi levels 3 - 5 are on
        serviceLevels = report.getServiceRunlevelStatus("acpid", 3, 5)
        isAcpiDisabledInRunLevel = True
        for level in serviceLevels:
            if serviceLevels[level]:
                isAcpiDisabledInRunLevel = False
                break;

        clusterServiceStatus = self.__getClusterNodeServicesStatus(report, distroRelease.getMajorVersion())
        clusterNode = ClusterNode(pathToClusterConfFile, distroRelease, report.getUname(), report.getHostname(),  
                                  report.getUptime(), report.generateNetworkMaps(), clusterServiceStatus, pathToClusterCommands, 
                                  report.getPackageVersion(ClusterNodes.PACKAGE_LIST),
                                  report.getPackageVersion(ClusterNodes.MODULE_LIST),
                                  report.isAcpiDisabledinCmdLine(), isAcpiDisabledInRunLevel)

        # Now append the fully formed object to the node
        self.__clusterNodes.append(clusterNode)
        return True
        

    # #######################################################################
    # Private helper methods for functions
    # #######################################################################
    def __getClusterNodeServicesStatus(self, report, distroMajorVersion) :
        """
        Returns an array of dictionaries of dictionaries that list the
        status of each runlevel for a service.

        Example:
        [{'cman': {3: False, 4: False, 5: False}}, {'clvmd': {}}, {'gfs': {}}, {'rgmanager': {3: False, 4: False, 5: False}}]

        @return: Returns an array of dictionaries.
        @rtype: Array

        @param report: A SReport object.
        @type report: SReport
        @param distroMajorVersion: The major version of a RHEL distro.
        @type distroMajorVersion: Int
        """
        chkconfigData = []
        # Set Distro dependent variables
        if (distroMajorVersion  == 5):
            clusterServices = ClusterNodes.RHEL5_CLUSTER_SERVICES
        elif (distroMajorVersion  == 4):
            clusterServices = ClusterNodes.RHEL4_CLUSTER_SERVICES
        else:
            return chkconfigData

        services = clusterServices
        sortedServiceKeys = services.keys()
        sortedServiceKeys.sort()
        for key in sortedServiceKeys:
            serviceDict = {}
            serviceDict[services.get(key)] = report.getServiceRunlevelStatus(services.get(key), 3, 5)                                                                             
            chkconfigData.append(serviceDict)
        return chkconfigData

