#!/usr/bin/env python
"""
This plugin will open yakuake sessions for each sosreport with
dbus. The plugin does the following:

1) In the current active session of yakuake it will cd to the working
directory that contains all the extracted plugins 

2) Open a new yakuake session for each report and then open the
var/log/messages file with less.

In order to use this plugin the "yakuake" has to be installed
dbus/messagebus has to be running($ service messagebus start).

@author    :  Shane Bradley
@contact   :  sbradley@redhat.com
@version   :  2.03
@copyright :  GPLv2
"""
import logging
import sys
import os.path
import dbus

import sx
import sx.plugins
from sx.logwriter import LogWriter

class Yakuake(sx.plugins.PluginBase):
    """
    This is a plugin for opening yakuake sessions.
    """
    def __init__(self, pathToPluginReportDir=""):
        """
        This init takes the root path to where the reports will be
        written. The parent class will then create the correct
        directory structure for the plugin.

        @param pathToPluginReportDir: This is the root path to where
        the report files will be written.
        @type pathToPluginReportDir: String
        """
        sx.plugins.PluginBase.__init__(self, "Yakuake", 
                                       "This plugin opens new tab sessions for each sysreport/sosreport and the dir that contains the reports.",
                                       ["Sosreport", "Sysreport"], False, True, 
                                       {"view_msglog": "Opens up /var/log/messages for each report in tab(options: on/off).",
                                        "fileviewer": "The path to the program that will be used to open the file."},
                                       pathToPluginReportDir)


        self.__pathToReportsRoot = ""
        self.__pathToReports = {}

        # Set default options;
        self.setOptionValue("view_msglog", "on");
        self.setOptionValue("fileviewer", "/usr/bin/less");

    # #######################################################################
    # Functions that should be overwritten in the plugin
    # #######################################################################
    def setup(self, reports) :
        """
        This function will setup data structure to hold any data/path
        to files that are needed to use in this plugin. 

        @param reports: This is the list of Report Objects.  
        @type reports: Array 
        """
        message = "Running setup for plugin: %s" %(self.getName())
        logging.getLogger(sx.MAIN_LOGGER_NAME).log(LogWriter.STATUS_LEVEL, message)
        for report in reports:
            if (self.isValidReportType(report)) :
                reportPath = report.getPathToExtractedReport()
                if (os.path.exists(reportPath)):
                    self.__pathToReports[report.getHostname()] = reportPath
                    if (len(self.__pathToReportsRoot) == 0) :
                        (head,tail) = os.path.split(reportPath) 
                        self.__pathToReportsRoot = head

    def action(self) :  
        """ 
        This function will open open yakuake konsole terminals tabs
        for each report.
        """
        message = "Yakuake will open tabs for each sosreport/sysreport /var/log/messages file."
        logging.getLogger(sx.MAIN_LOGGER_NAME).info(message)   
        try :
            bus = dbus.SessionBus()
            # Get the current active session
            yakuakeSessionsObj = bus.get_object("org.kde.yakuake","/yakuake/sessions")
            yakuakeSessionsInterface = dbus.Interface(yakuakeSessionsObj, "org.kde.yakuake")
            # Get terminalID of current session, so we can
            # jump back after new sessions are created. 
            terminalID = yakuakeSessionsInterface.activeTerminalId()
            # Run command to cd to the reports directory
            yakuakeSessionsInterface.runCommandInTerminal(terminalID, "cd %s" % (self.__pathToReportsRoot))
            keys = self.__pathToReports.keys()
            keys.sort()
            for key in keys:
                pathToReport = self.__pathToReports[key]
                if (os.path.exists(pathToReport)):
                    try:
                        # Add a new tab
                        yakuakeSessionsInterface.addSession()

                        # Get terminal/session ID of new tab created since it is active.
                        activeTerminalID = yakuakeSessionsInterface.activeTerminalId()
                        activeSessionID = yakuakeSessionsInterface.activeSessionId()

                        # Run command on that terminal ID
                        yakuakeSessionsInterface.runCommandInTerminal(activeTerminalID, "cd %s" % (pathToReport))
                        pathToMessageFile = os.path.join(pathToReport, "var/log/messages")    
                        if (os.path.exists(pathToMessageFile) and ("on" == self.getOptionValue("view_msglog"))) :
                            fileViewer = self.getOptionValue("fileviewer")
                            if (os.path.exists(fileViewer)):
                                yakuakeSessionsInterface.runCommandInTerminal(activeTerminalID, 
                                                                              "%s %s" % (fileViewer, pathToMessageFile))
                            else:
                                message = "The file viewer does not exist: %s" %(fileViewer)
                                logging.getLogger(sx.MAIN_LOGGER_NAME).error(message)
                        # Set Tab title
                        currentTabObj = bus.get_object("org.kde.yakuake","/yakuake/tabs")
                        currentTabInterface = dbus.Interface(currentTabObj, "org.kde.yakuake")
                        currentTabInterface.setTabTitle(activeSessionID, key)
                    except dbus.exceptions.DBusException, e:
                        message = "An error occurred trying to communicate with dbus to create/modify a session."
                        logging.getLogger(sx.MAIN_LOGGER_NAME).error(message)
                else:
                    message = "The report directory does not exist: %s" % (pathToReport)
                    logging.getLogger(sx.MAIN_LOGGER_NAME).error(message)
            # Change back to the orginal session in yakuake
            yakuakeSessionsInterface.raiseSession(terminalID)
        except dbus.exceptions.DBusException, e:
            message = "An error occurred trying to communicate with dbus to get current session before adding sessions."
            logging.getLogger(sx.MAIN_LOGGER_NAME).error(message)
            

