#!/usr/bin/env python
"""
This class will extract the different types of reports.

@author    :  Shane Bradley
@contact   :  sbradley@redhat.com
@version   :  2.03
@copyright :  GPLv2
"""
import os
import os.path
import string
import sys
import shutil
import re
import logging

import sx
from sx.logwriter import LogWriter
from tarball import TarBall

class ReportExtractor:
    """
    This class will extract all the different types of reports.
    """
    def __init__(self, compressedReportPath, extractedReportPath, reportClasses):
        """
        @param compressedReportPath: The path to the location where
        the compressed reports will be kept.
        @type compressedReportPath: String
        @param extractedReportPath: The path to the location where
        the extracted reports will be kept.
        @type extractedReportPath: String
        @param reportClasses: An array of report class objects.
        @type reportClasses: Array
        """
        self.__compressedReportPath = compressedReportPath
        self.__extractedReportPath = extractedReportPath
        self.__reportClasses = reportClasses

        # The paths to the compressed and extracted reports should
        # already exists and if they do not then we need to exit.
        try:
            if (not os.access(self.__compressedReportPath, os.F_OK)):
                message = "The path to compressed reports does not exist: %s." % (self.__compressedReportPath)
                logging.getLogger(sx.MAIN_LOGGER_NAME).error(message)
                sys.exit(2)
        except IOError:
            message = "There was an error accessing the  path to the compressed reports: %s." % (self.__compressedReportPath)
            logging.getLogger(sx.MAIN_LOGGER_NAME).error(message)
            sys.exit(2)
        try:
            if (not os.access(self.__extractedReportPath, os.F_OK)):
                message = "The path to extracted reports does not exist: %s." % (self.__extractedReportPath)
                logging.getLogger(sx.MAIN_LOGGER_NAME).error(message)
                sys.exit(2)
        except IOError:
            message = "There was an error accessing the  path to the extracted reports: %s." % (self.__extractedReportPath)
            logging.getLogger(sx.MAIN_LOGGER_NAME).error(message)
            sys.exit(2)

    # ##########################################################################
    # Get functions
    # ##########################################################################
    def getCompressedReportPathDir(self):
        """
        Returns a string representation of path to compressed
        sreports.

        @return: Returns a string representation of path to compressed
        sreports.
        @rtype: String
        """
        return self.__compressedReportPath

    def getExtractedReportPathDir(self):
        """
        Returns a string representation of path to extracted sreports.
        
        @return: Returns a string representation of path to extracted
        sreports.
        @rtype: String
        """
        return self.__extractedReportPath

    # ##########################################################################
    # Helper functions
    # ##########################################################################
    def __findReportClass(self, pathToReportFile) :
        """
        Returns the class that matches the report file.

        @return: Returns the class for that matches the report.
        @rtype: Class
        
        @param pathToReportFile: Path to Report file to check.
        @type pathToReportFile: String
        """
        if (os.path.isfile(pathToReportFile)) :
            data = TarBall.list(pathToReportFile) 
            if (len(data) > 0) :
                # Build the expression for all the different signature
                # files inside that report file.
                regex = ""
                for reportClass in self.__reportClasses:
                    # Escape out . with \.
                    if (len(regex) > 0) :
                        regex += "|"
                        regex += "(?P<%s>.*%s.*)" %(reportClass.__name__,
                                                    reportClass.TYPE_DETECTION_FILE.replace(".", "\."))
                rem = re.compile(regex)
                for line in data:
                    mo = rem.match(line)
                    if mo:
                        for reportClass in self.__reportClasses:
                            if (line.find(reportClass.TYPE_DETECTION_FILE) >= 0) :
                                return reportClass
        return None

    # ##########################################################################
    # Extractor that will extract reports and find the type
    # ##########################################################################
    def load(self):
        """
        Returns the list of report paths that have already been
        extracted based on the path of extractedReportPath pass to
        init.

        @return: Returns the list of report paths that have already
        been extracted based on the path of extractedReportPath pass
        to init.  
        @rtype: Array
        """
        message = "Loading all the known report types for previously extracted reports."
        logging.getLogger(sx.MAIN_LOGGER_NAME).log(LogWriter.STATUS_LEVEL, message)
        
        validReportArray = []
        for filename in os.listdir(self.__extractedReportPath):
            if (not filename == "reports"):
                src = os.path.join(self.__extractedReportPath, filename)
                reportClass = self.__findReportClass(src)
                if (reportClass == None) :
                    message = "The report type could not be determined: %s" %(src)
                    logging.getLogger(sx.MAIN_LOGGER_NAME).warning(message)
                    continue
                report = reportClass()
                if (not report == None) :
                    report.setPathToExtractedReport(src)
                    report.setPathToCompressedReport(src.replace("ereports", "creports")) 
                    validReportArray.append(report)
        return validReportArray

    def extract(self, reportFileArray) :
        """
        This function will extract all the reports in the array if
        they are a known type. It will return a list of paths to valid
        reports that were extracted.

        @return: Returns a list of all the reports that were
        successfully extracted.
        @rtype: Array

        @param reportFileArray: An array of all the reports to attempt
        extract. If there is not a known type then the report will not
        be extracted and moved to archive location.
        @type reportFileArray: Array
        """
        reportFileArray.sort()
        validReportArray = []
        if (not len(reportFileArray) > 0):
            return validReportArray
        
        message = "The following registered report types will be used to extract the reports:"
        for reportClass in self.__reportClasses:
            message += "\n\t  %s" %(reportClass.__name__)
        
        logging.getLogger(sx.MAIN_LOGGER_NAME).log(LogWriter.STATUS_LEVEL, message)
        message = "This could take a while if the files are large in size. BE PATIENT."
        logging.getLogger(sx.MAIN_LOGGER_NAME).info(message)

        for src in reportFileArray:
            (head, tail) =  os.path.split(src) 
            dst = os.path.join(self.__compressedReportPath, tail)
            reportClass = self.__findReportClass(src)
            if (reportClass == None) :
                message = "The file will be skipped since unable to recongize the report type: %s" %(src)
                logging.getLogger(sx.MAIN_LOGGER_NAME).warning(message)
                continue
            try:
                shutil.move(src, dst)
                report = reportClass()
                if (not report == None) :
                    if report.extract(dst, self.__extractedReportPath):
                        validReportArray.append(report)
            except (IOError, os.error):
                message = "Cannot move %s to %s " %(src, dst)
                logging.getLogger(sx.MAIN_LOGGER_NAME).error(message)
        validReportArray.sort()
        return validReportArray


