#!/usr/bin/env python
"""
This class is a container for different kind of reports. This is the
base class that all report types should inherit.

@author    :  Shane Bradley
@contact   :  sbradley@redhat.com
@version   :  2.03
@copyright :  GPLv2
"""
import string
import os
import os.path
import shutil
import re
import logging

import sx
from sx.logwriter import LogWriter
from sx.tarball import TarBall

class Report: 
    """
    This class is a container for different kind of reports. This is
    the base class that all report types should inherit.
    """
    def __init__(self, name, description, stripDirectoriesDepth=1) :
        """
        @param name: The name of the report.
        @type name: String
        @param description: A description of the report.
        @type description: String
        @param stripDirectoriesDepth: This value will strip the root
        directories of the report to a depth that is given. Default is 1.
        @type stripDirectoriesDepth: Int
        """
        self.__name = name
        self.__description = description
        # The Default is 1 for how many directories need to be
        # removed.
        self.__stripDirectoriesDepth = stripDirectoriesDepth

        self.__pathToReportFile = ""
        self.__pathToExtractedReport = ""
        self.__pathToTmpExtractedReport = ""

    def __str__(self) :
        """
        Returns a formatted string of this object.
        
        @return: Returns a formatted string of this object.
        @rtype: String
        """
        return "%s: %s" %(self.getName(), self.getDescription())

    def getName(self) :
        """
        Returns name of the report.

        @return: Returns name of the report.
        @rtype: String
        """
        return self.__name

    def getDescription(self) :
        """
        Returns description of the report.

        @return: Returns description of the report.
        @rtype: String
        """
        return self.__description

    def getPathToTarFile(self) :
        """
        This function returns the path to the archive report file that
        will be extracted.

        @return: Returns the path to the archive report file that will
        be extracted.
        @rtype: String
        """
        return self.__pathToReportFile

    def getPathToExtractedReport(self) :
        """
        This function returns the path to the extracted report.
        
        @return: Returns the path to the extracted report.
        @rtype: String
        """
        return self.__pathToExtractedReport

    def getPathToTmpExtractedReport(self) :
        """
        This function returns the temporary path to the extracted report.
        
        @return: Returns the temporary path to the extracted report.
        @rtype: String
        """
        return self.__pathToTmpExtractedReport

    def getType(self) :
        """
        This function returns the class name of this report.

        @return: The class name of the report.
        @rtype: String
        """
        return self.__class__.__name__

    def setPathToExtractedReport(self, pathToExtractedReport):
        """
        This function sets the path to the extracted report
        directory.
        
        @param pathToExtractedReport: The path to the extracted
        report directory.
        @type pathToExtractedReport: String
        """
        self.__pathToExtractedReport = pathToExtractedReport
        (head, tail) = os.path.split(self.__pathToExtractedReport)
        self.__pathToTmpExtractedReport = os.path.join(head, ".%s" %(tail))

    def setPathToCompressedReport(self, pathToCompressedReport):
        """
        This function sets the path to the compressed report
        directory.
        
        @param pathToCompressedReport: The path to the compressed
        report directory.
        @type pathToCompressedReport: String
        """
        self.__pathToReportFile = pathToCompressedReport

    def clean(self) :
        """
        Remove the temporary location of files that were copied from
        extracted report.
        """
        if os.path.exists(self.__pathToTmpExtractedReport): 
            shutil.rmtree(self.__pathToTmpExtractedReport)

    def __copy(self, src, dst) :
        """
        Returns the path to the file that was copied from src to dst
        path. Empty string is returned if no file was copied because
        of an error.

        This function makes a temporary copy of the file, it does not
        return the actual path to the extracted report. This is an
        extact duplicate of the extracted report. The temporary
        directory of files is dynamically created as requests are
        made. If file is already copied then it does not create it
        again.

        @return: Returns the path to the file that was copied from
        src to dst path. None is returned if copy was not completed.
        @rtype: String

        @param src: The path to the source file to copy.
        @type src: String
        @param dst: The path to the destination file to copy src file
        to.
        @type dst: String

        """
        if (os.path.isfile(src)) : 
            pDir,filename = os.path.split(dst)
            try:
                if not os.access(pDir, os.F_OK):
                    os.makedirs(pDir)
            except (IOError, os.error):
                message =  "Cannot create directory: %s" % (pDir)
                logging.getLogger(sx.MAIN_LOGGER_NAME).error(message)
                return ""
            try:
                # If file already exist then do not overwrite it, just
                # return the dst.
                if not (os.path.isfile(dst)) :
                    shutil.copyfile(src, dst)
                return dst 
            except (IOError, os.error):
                message = "IOerror occured copying the file %s to destination \n\t%s" % (src, dst) 
                logging.getLogger(sx.MAIN_LOGGER_NAME).error(message)
                return ""
        elif (os.path.isdir(src)):
            try:
                if not os.access(dst, os.F_OK):
                    os.makedirs(dst)
            except (IOError, os.error):
                message =  "Cannot create directory: %s" %(pDir)
                logging.getLogger(sx.MAIN_LOGGER_NAME).error(message)
                return ""
            for f in os.listdir(src):
                srcPath = os.path.join(src, f) 
                dstPath = os.path.join(dst, f) 
                self.__copy(srcPath, dstPath) 
            return dst
        # Return None if file/dir does not exist
        return ""

    # ##########################################################################
    # Extract File/Data from extracted sreports functions
    # ##########################################################################
    def extractData(self, pathToFile) :
        """
        This function will return the data in an array. Where each
        newline in file is a seperate item in the array. This should
        really just be used on relatively small files.

        None is returned if no file is found.

        @return: Returns an array of Strings, where each newline in
        file is an item in the array.
        @rtype: Array

        @param pathToFile: The path to the file, which is relative to
        the root report directory.
        @type pathToFile: String        
        """
        pathToFile = self.extractPath(pathToFile)
        if (len(pathToFile) > 0) :
            try:
                fin = open(pathToFile, "r")
                data = fin.readlines()
                fin.close()
                return data
            except (IOError, os.error):
                message = "IOerror occured reading the file -> %s." %(pathToFile)
                logging.getLogger(sx.MAIN_LOGGER_NAME).error(message)
            else:
                fin.close()
        return None

    def extractPath(self, pathToFile):
        """
        This function will return the path to the temporary file. If
        file does not exist then empty string is returned.
        
        @return: Returns the path to the temporary file. Empty string
        is returned if no file is found.

        @param pathToFile: The path to the file, which is relative to
        the root report directory.
        @type pathToFile: String        
        """
        if (len(pathToFile) > 0):
            src = os.path.join(self.__pathToExtractedReport, pathToFile)
            dst = os.path.join(self.__pathToTmpExtractedReport, pathToFile)
            return self.__copy(src,dst)
        return ""

    def extract(self, pathToReportFile, extractDir):
        """
        This function will extract the report to the extract
        dir. Currently all reports have to be a tarfile archive.

        This function should be overwrriten with the specific report
        type extraction function. Returns True if there is not fatal
        errors.

        This function should be overridden by all report objects that
        inherit this class if they want the extraction directory to be
        called something other than what is contained in the report. 

        @return: Returns True if there was no fatal errors. TarBall
        errors are ignored because they should not be fatal, if they
        are the child should catch the errors since dir will not
        exist. If there are fatal errors, then False is returned.
        @rtype: boolean

        @param pathToReportFile: This is the path to the report file.
        @type pathToReportFile: String
        @param extractDir: The full path to directory for extraction.
        @type extractDir: String
        """
        self.__pathToReportFile = pathToReportFile
        message = "Extracting the %s: %s" %(self.getName(), self.getPathToTarFile())
        logging.getLogger(sx.MAIN_LOGGER_NAME).log(LogWriter.STATUS_LEVEL, message)
        if (not len(extractDir) > 0):
            return False
        # Check for duplicate extraction point and rename if it exists.
        if os.path.exists(extractDir) :
            for i in range(1, 10) :
                (head, tail) = os.path.split(extractDir) 
                duplicatePath = os.path.join(head, "%s-duplicate_%s" %(tail, str(i)))
                if (not os.path.exists(duplicatePath)) :
                    # Directory does not exist so we can extract to this path
                    extractDir = duplicatePath
                    break;
        # Set path to extraction point and temporary directory
        self.__pathToExtractedReport = extractDir
        (head, tail) = os.path.split(self.__pathToExtractedReport)
        self.__pathToTmpExtractedReport = os.path.join(head, ".%s" %(tail))
        # Do the extraction of the tar file
        try:
            if not os.access(self.__pathToExtractedReport, os.F_OK):
                os.makedirs(self.__pathToExtractedReport)
        except (IOError, os.error):
            message =  "IO error occured on creating the directory: \n\t %s" %(self.__pathToExtractedReport)
            logging.getLogger(sx.MAIN_LOGGER_NAME).error(message)
            return False
        TarBall.extract(self.__pathToExtractedReport, self.getPathToTarFile(), self.__stripDirectoriesDepth) 
        return True


