#!/usr/bin/env python
"""
This file contains various containers for working with
sosreports/sysreports reports.

This file contains the container for sosreport report object.

@author    :  Shane Bradley
@contact   :  sbradley@redhat.com
@version   :  2.03
@copyright :  GPLv2
"""
import string
import os.path
import logging
import re

import sx
from sx.reports import Report
from sx.tarball import TarBall

class NetworkMap:
    """
    Container for network information for network information.
    """
    def __init__(self, interface, hwAddr, ipAddr, subnetMask, etcHostsMap):
        """
        @param interface: The network interface.
        @type interface: String
        @param hwAddr: The hardware address
        @type hwAddr: String
        @param ipAddr: The ip address.
        @type ipAddr: String
        @param subnetMask: The subnet mask.
        @type subnetMask: String
        @param etcHostsMap: This is a dictionary of all ip/hostname
        entries in /etc/hosts file.
        @type etcHostsMap: Dictionary
        """
        self.__interface = interface
        self.__hwAddr = hwAddr
        self.__ipAddr = ipAddr
        self.__subnetMask = subnetMask
        self.__etcHostsMap = etcHostsMap

        self.__hostnames = etcHostsMap.get(self.__ipAddr) 
        if (self.__hostnames == None):
            self.__hostnames = []

    def __str__(self) :
        """
        This function returns a string reprenstation of the object.
        
        @return: Returns a string reprenstation of the object.
        @rtype: String
        """
        returnString = ""
        returnString += "interface:  %s\n" % (self.getInterface())
        returnString += "ip address: %s\n" % (self.getIPAddress())
        returnString += "hw address: %s\n" % (self.getHardWareAddress())
        returnString += "hostnames:  " 
        for hostname in self.getHostnames() :
            returnString += "%s " %(hostname)
        returnString += "\n"
        returnString += "hostnames:  %s\n" % (str(self.getHostnames()))
        return returnString

    def getInterface(self):
        """
        Returns the network interface.

        @return: Returns the network interface.
        @rtype: String
        """
        return self.__interface

    def getHardWareAddress(self) :
        """
        Returns the Hardware address of this interface.

        @return: Returns the Hardware address of this interface.
        @rtype: String
        """
        return self.__hwAddr

    def getIPAddress(self):
        """
        Returns the IP address of this interface.

        @return: Returns the IP address of this interface.
        @rtype: String
        """
        return self.__ipAddr

    def getSubnetMask(self) :
        """
        Returns the subnet mask of this interface.

        @return: Returns the subnet mask of this interface.
        @rtype: String
        """
        return self.__subnetMask

    def getHostnames(self) :
        """
        Returns an array of all the hostnames assoicated with this
        interface.

        @return: Returns an array of all the hostnames assoicated with
        this interface.
        @rtype: Array
        """
        return self.__hostnames

    def getEtcHostsMap(self):
        """
        This returns a map of all the entries in /etc/hosts. The key
        is the ip and value is list of hostnames.

        @return: Returns a dictionary of all the entries in
        /etc/hosts.
        @rtype: Dictionary
        """
        return self.__etcHostsMap

    def hasHostnameMapped(self, hostname) :
        for key in self.getEtcHostsMap().keys():
            if (hostname in self.getEtcHostsMap().get(key)):
                return True
        return False

class DistroRelease:
    """
    This is a container for the information that is in the
    /etc/redhat-release file.
    """
    def __init__(self, distroName, distroType, distroMajorVersion, distroMinorVersion):
        """
        @param distroName: The name of the distro.
        @type distroName: String
        @param distroType: The type of distro.
        @type distroType: String
        @param distroMajorVersion: The major version of the distro.
        @type distroMajorVersion: Int
        @param distroMinorVersion: The minor version of the distro.
        @type distroMinorVersion: Int
        """
        if (distroName == "Red Hat Enterprise Linux"):
            distroName = "RHEL"
        self.__distroName = distroName
        self.__distroType = distroType
        self.__distroMajorVersion = int(distroMajorVersion)
        self.__distroMinorVersion = int(distroMinorVersion)

    def __str__(self) :
        """
        This function returns a string reprenstation of the object.
        
        @return: Returns a string reprenstation of the object.
        @rtype: String
        """
        return "%s %s %s.%s" %(self.__distroName, self.__distroType, self.__distroMajorVersion, self.__distroMinorVersion)

    def getDistroName(self) :
        """
        Returns the name of the distro release.
        
        @return: Returns the name of the distro release.
        @rtype: String
        """
        return self.__distroName

    def getDistroType(self) :
        """
        Returns the type of the distro release. 
        For example: AS,ES,WS,SERVER,etc.
        
        @return: Returns the type of the distro release.
        @rtype: String
        """
        return self.__distroType

    def getMajorVersion(self) :
        """
        Returns the major version of the distro release.
        
        @return: Returns the major version of the distro release.
        @rtype: String
        """
        return self.__distroMajorVersion

    def getMinorVersion(self) :
        """
        Returns the minor version of the distro release.
        
        @return: Returns the minor version of the distro release.
        @rtype: String
        """
        return self.__distroMinorVersion

class SReport() :
    def getRedhatReleaseVersion(self) :
        """
        Returns the release version from the release file(rhel:
        /etc/redhat-release).
        
        Example from /etc/redhat-release:
        "Red Hat Enterprise Linux AS release 4 (Nahant Update 6)"
        "Red Hat Enterprise Linux Server release 5.3 (Tikanga)"
        "Fedora release 10 (Cambridge)"
        
        @return: Returns a DistroRelease object that contains information
        about the reports Distrobution Release.
        @rtype: DistroRelease
        """
        pathToReleaseFile = self.extractPath("etc/redhat-release")
        try:
            fin = open(pathToReleaseFile, "r")
            data = fin.read()
            fin.close()
            regex = "^(?P<distroName>Red Hat Enterprise Linux|Fedora) " + \
                "(?P<distroType>AS|ES|WS|Server|\s)?\ ?" + \
                "release (?P<distroMajorVersion>\d{1,2})[\. ]?" + \
                "(?P<distroMinorVersion1>\d{1,})?\s*" + \
                "(?P<isbeta>Beta)?\s*" + \
                "\(\D*(?P<distroMinorVersion2>\d{1,})?\)"

            rem = re.compile(regex)
            mo = rem.match(data)
            if mo:
                distroName = mo.group("distroName")
                distroType =  mo.group("distroType")
                distroMajorVersion = mo.group("distroMajorVersion")
                if (distroMajorVersion == None):
                    distroMajorVersion = 0
                distroMinorVersion =  mo.group("distroMinorVersion1")
                if (distroMinorVersion == None):
                    if (mo.group("distroMinorVersion2") == None):
                        distroMinorVersion = 0 
                    else:
                        distroMinorVersion = mo.group("distroMinorVersion2")
                return DistroRelease(distroName, distroType, distroMajorVersion, distroMinorVersion) 
        except (IOError, os.error):
            message = "IOerror occured reading the file:\n\t%s." % (pathToReleaseFile)
            logging.getLogger(sx.MAIN_LOGGER_NAME).error(message)
        except (TypeError) :
            message = "IOerror occured reading the file:\n\t%s." % (pathToReleaseFile)
            logging.getLogger(sx.MAIN_LOGGER_NAME).error(message)
        return None

    def getPackageVersion(self, packageList) :
        """
        Matches the package in packageList to an re.matchobject. If
        match is found then that item is added to the dictionary. The
        dictionary uses the array item as key and package that is
        found in installed-rpms file as the value.
        
        If the key has empty string as value then no package was
        found.
        
        All "-" will be replaced with "__" since "-" is not allowed in
        the variable name using the "?P<var>" regex.
    
        @return: Returns Dictionary of the matched packages.
        @rtype: Dictionary
    
        @param packageList: An array of strings that will be
        searched for in installedrpms file.
        @type packageList: Array
        """
        installedrpms = self.getPathToInstalledRPMS()
        data = []
        try:
            fin = open(installedrpms, "r")
            data = fin.readlines()
            fin.close()
        except (IOError, os.error):
            message = "IOerror occured reading the file -> %s" % installedrpms
            logging.getLogger(sx.MAIN_LOGGER_NAME).error(message)
            return {}

        packageVersionDict = {}
        combindedRegex = ""
        # Build a regex for all packages so all packages are searched.
        for package in packageList:
            # Populate the dictionary with keys which is item in array
            packageVersionDict[package] = []
            # Build the regex
            if (not len(combindedRegex) == 0):
                combindedRegex += "|" 
            # reencode the "-" to "__" so regex be happy
            combindedRegex += "(?P<%s>%s-\d.*)" %(package.replace("-","__"), package)

        # Compile the regex to start matching
        rem = re.compile(r"%s" % combindedRegex)
        # Now search the data
        for line in data :
            mo = rem.match(line)
            if mo:
                moDict = mo.groupdict() 
                for key in moDict.keys():
                    if (not moDict[key] == None):
                        # Dont include the date of installed
                        packageVersionDict[key.replace("__", "-")].append(moDict[key].split(" ", 1)[0].strip())

        # Remove packages that have empty array which means package was not found.
        keys = packageVersionDict.keys() 
        for key in keys:
            if (not len(packageVersionDict[key]) > 0):
                del packageVersionDict[key]
        return packageVersionDict

    def getServiceRunlevelStatus(self, serviceName, runlevelMin, runlevelMax) :
        """
        Returns a dictionary for each for status of the service. An
        empty dictionary is returned if service is not in the file. If
        the service exist in the string then a dictionary is returned
        where the key is an int for runlevel and boolean is value for
        service status(True is enabled, False is disabled).
    
        @return: Returns a dictionary for each for status of the service. An
        empty dictionary is returned if service is not in the file. If
        the service exist in the string then a dictionary is returned
        where the key is an int for runlevel and boolean is value for
        service status(True is enabled, False is disabled).
        @rtype: Dictionary
    
        @param serviceName: Name of service.
        @type serviceName: String
        @param runlevelMin: The runlevel to start search on.
        The range is (1 -6).
        @type runlevelMin: Int
        @param runlevelMax: The runlevel to stop search on.
        The range is (0 - 6).
        @type runlevelMax: Int
        """
        # Check multiple paths for correct file
        pathToChkConfig = self.extractPath("chkconfig")
        if (not len(pathToChkConfig) > 0):
            pathToChkConfig = self.extractPath("sos_commands/startup/chkconfig_--list")
        if (not len(pathToChkConfig) > 0) :
            return  {}

        chkConfigData = []
        servicesDict = {}
        if (pathToChkConfig == None) :
            return servicesDict
        try:
            fin = open(pathToChkConfig, "r")
            chkConfigData = fin.readlines()
            fin.close()
        except (IOError, os.error):
            message = "IOerror occured reading the file: %s." % pathToChkConfig
            logging.getLogger(sx.MAIN_LOGGER_NAME).error(message)
            return servicesDict
        else:
            fin.close()
        if (not len(chkConfigData) > 0) :
            return servicesDict
        # make sure min is in range 0 - 6
        if (runlevelMin < 0) :
            runlevelMin = 0
        if (runlevelMin > 6) :
            runlevelMin = 6
        # make sure max is in range 0 - 6
        if (runlevelMax < 0) :
            runlevelMax = 0
        if (runlevelMax > 6) :
            runlevelMax = 6
        splitServiceLine = []

        # this re compile string will split the line like the following(\t tabs between levels):
        # line          -->acpid          0:off     1:off   2:off    3:on   4:on  5:on   6:off
        # regex grouped --> ('acpid', '0:off', '1:off', '2:off', '3:on', '4:on', '5:on', '6:off')
        # group[0] is the runlevel string {0,1,2,3,4,5,6}
        # group[1] is the enabled/disabled string {on,off} on is True and off is False
        rec = re.compile( r'^(%s)\s*([0-6].o[f|n][f]?)\s*([0-6].o[f|n][f]?)\s*([0-6].o[f|n][f]?)\s*([0-6].o[f|n][f]?)\s*([0-6].o[f|n][f]?)\s*([0-6].o[f|n][f]?)\s*([0-6].o[f|n][f]?).*$' %(serviceName) )

        for line in chkConfigData:
            mo = rec.match(line) 
            if mo:
                for group in mo.groups() :
                    splitGroup = group.split(":")
                    if (len(splitGroup) == 2) :
                        if ((int(splitGroup[0]) >= runlevelMin) and
                            (int(splitGroup[0]) <= runlevelMax)):
                            if splitGroup[1] == "on":
                                servicesDict[int(splitGroup[0])] =  True
                            elif splitGroup[1] == "off":
                                servicesDict[int(splitGroup[0])] =  False
        return servicesDict

    def isAcpiDisabledinCmdLine(self) :
        """
        This function returns True if "noacpi" or "acpi=off" is in
        /proc/cmdline for the ClusterNode.
        
        @return: This function returns True if "noacpi" or "acpi=off"
        is in /proc/cmdline for the ClusterNode.
        @rtype: Boolean
        """
        pathToProcCmdline = self.extractPath("proc/cmdline")
        if (pathToProcCmdline == None) :
            message = "The file does not exist: %s." % pathToProcCmdline
            logging.getLogger(sx.MAIN_LOGGER_NAME).error(message)
            return False
        data = ""
        try:
            fin = open(pathToProcCmdline, "r")
            data = fin.readline()
            fin.close()
        except (IOError, os.error):
            message = "IOerror occured reading the file: %s." % pathToProcCmdline
            logging.getLogger(sx.MAIN_LOGGER_NAME).error(message)
            return False
        if ((string.find(data, "noacpi") >=0) or
            (string.find(data, "acpi=off") >=0)) :
            return True
        return False

    # ###########################################################################
    # Functions to generate Network map of sreport 
    # ###########################################################################
    def __generateEtcHostsMap(self): 
        """
        This function generates a map of all the ip to hostnames in
        the /etc/hosts file.
        
        The key is the ip and its value is an array of hostnames.

        @return: Returns a dictionary of the ip to hostnames in the
        file /etc/hosts.
        @rtype: Dictionary
        """
        pathToEtcHosts = self.extractPath("etc/hosts")
        etcHostsMap = {} 
        lines = []
        try:
            fin = open(pathToEtcHosts, "r")
            lines = fin.readlines()
            fin.close()
        except (IOError, os.error):
            message = "An i/o error occured in reading the file getting the file: \n\t%s" %(pathToEtcHosts)
            logging.getLogger(sx.MAIN_LOGGER_NAME).error(message)
            return etcHostsMap

        ipRegex = "(?:[\d]{1,3})\.(?:[\d]{1,3})\.(?:[\d]{1,3})\.(?:[\d]{1,3})"
        rem = re.compile("(%s)\s(.*)" %(ipRegex)) 
        for line in lines:
            mo = rem.match(line)
            if mo:
                ipAddress = mo.group(1).strip()
                hostnames = mo.group(2).strip().split()
                etcHostsMap[ipAddress] = hostnames
        return etcHostsMap

    def generateNetworkMaps(self) :
        """
        This function generates a NetworkMap of all the interfaces. It
        will return an array of NetworkMap Objects. Loopback and sitX
        will not be mapped.

        @return: Returns an array of NetworkMap objects.
        @rtype: Array
        """
        pathToIfconfig = self.extractPath("ifconfig")
        lines = []
        try:
            fin = open(pathToIfconfig, "r")
            lines = fin.readlines()
            fin.close()
        except (IOError, os.error):
            message = "An i/o error occured in reading the file getting the file: \n\t%s" %(pathToIfconfig)
            logging.getLogger(sx.MAIN_LOGGER_NAME).error(message)
            return []

        # Generate a map of /etc/hosts contents
        etcHostsMap = self.__generateEtcHostsMap()

        remIface = re.compile ("(.*)Link encap:.*\s\sHWaddr\s([0-9a-zA-Z\:].*)")
        remAddr  = re.compile(".*inet addr:([\d\.]*)\s.*Mask:([\d\.]*)")
        networkMaps = []
        for index in range(0, len(lines)):
            line = lines[index].strip()
            moIface = remIface.match(line)
            if moIface:
                interface = moIface.group(1).strip()
                hwAddr = moIface.group(2).strip()
                # Get the next line which is the ip/subnet line
                line = lines[index + 1].strip()
                moAddr  = remAddr.match(line)
                if moAddr:
                    ipAddr = moAddr.group(1).strip()
                    subnetMask = moAddr.group(2).strip()
                    # Append map to list of maps
                    networkMaps.append(NetworkMap(interface, hwAddr, ipAddr, subnetMask, etcHostsMap))
        return networkMaps

class Sosreport(sx.reports.Report, SReport) :
    """
    This class is a container for sosreport report object.

    @cvar TYPE_DETECTION_FILE: This is a path to file that can
    uniquely indentify the report object.
    @type TYPE_DETECTION_FILE: String
    """
    TYPE_DETECTION_FILE = "sos_logs/sos.log"
    def __init__(self) :
        sx.reports.Report.__init__(self, "sosreport",
                                   "A report object for sosreport files")
        self.__hostname = ""

    def extract(self, pathToReportFile, extractDir):
        """
        This function will extract the report to the extract
        dir. Currently all reports have to be a tarfile archive.

        This function overrides the parent function. The reason is
        that a new extractDir will be created. Then the parent extract
        function will be called.

        @return: Returns True if there was no fatal errors. TarBall
        errors are ignored because they should not be fatal, if they
        are the child should catch the errors since dir will not
        exist. If there are fatal errors, then False is returned.
        @rtype: boolean

        @param pathToReportFile: This is the path to the report file.
        @type pathToReportFile: String
        @param extractDir: The full path to directory for extraction.
        @type extractDir: String
        """
        fileContents = TarBall.getFileContents(pathToReportFile, "/sos_commands/kernel/uname_-a")
        if (len(fileContents) > 0):
            self.__hostname = string.strip(string.split(fileContents[0])[1])
        # Now call the parent function to finish the extraction
        sx.reports.Report.extract(self, pathToReportFile,
                                  os.path.join(extractDir, self.__hostname))
        if (os.path.exists(os.path.join(self.getPathToExtractedReport(),
                                        Sosreport.TYPE_DETECTION_FILE))):
            return True
        return False

    # ##########################################################################
    # Helper functions
    # ##########################################################################
    def getUname(self) :
        """
        This function will return a string of the "uname -a" data.

        @return: Returns a string of the "uname -a" data.
        @rtype String
        """
        unameAData = self.extractData("sos_commands/kernel/uname_-a")
        if (unameAData == None) :
            return ""
        return unameAData[0]
        
    def getUptime(self) :
        """
        This function will return a string of the "uptime" data.

        @return: Returns a string of the "uptime" data.
        @rtype String
        """
        uptime = self.extractData("uptime")
        if (uptime == None) :
            return ""
        return uptime[0].strip()

    def getArch(self) :
        """
        This function will return the arch for the report.

        @return: Returns the arch for the report.
        @rtype String
        """
        unameAData = self.getUname()
        regexArch = "(?P<arch>noarch|i386|i586|i686|ia64|ppc|s390|s390x|x86_64)"
        remArch = re.compile(regexArch, re.IGNORECASE)
        moArch = remArch.search(unameAData)
        if moArch:
            return moArch.group("arch")
        return ""

    def getHostname(self) :
        """
        This function will return a string that is the hostname from the
        "uname -a" data.

        @return: Returns a string that is the hostname from the "uname
        -a" data.
        @rtype String
        """
        if (not len(self.__hostname) > 0):
            # If hostname is not set or need from previous extracted report
            unameAData = self.getUname()
            if (not len(unameAData) > 0) :
                self.__hostname = "unknown_hostname"
            else:
                usplit = unameAData.split()
                if (len(usplit) >= 1):
                    self.__hostname = usplit[1]
                else:
                    self.__hostname = "unknown_hostname"
        return self.__hostname

    def getPathToInstalledRPMS(self) :
        """
        Returns the path to the file that contains the list of
        installed rpms.

        @returns: Returns the path to the file that contains the list
        of installed rpms.
        @rtype: String
        """
        pathToInstalledRPMS = self.extractPath("installed-rpms")
        if (not len(pathToInstalledRPMS) > 0):
            pathToInstalledRPMS = self.extractPath("sos_commands/rpm/rpm_-qa_--qf_NAME_-_VERSION_-_RELEASE_-_ARCH_INSTALLTIME_date_.b")
            if (not len(pathToInstalledRPMS) > 0):
                pathToInstalledRPMS = self.extractPath("sos_commands/rpm/rpm_-qa_--qf_NAME_-_VERSION_-_RELEASE_-_ARCH")
        return pathToInstalledRPMS

