#!/usr/bin/env python
"""
This class is a container for sysreport report object.

@author    :  Shane Bradley
@contact   :  sbradley@redhat.com
@version   :  2.03
@copyright :  GPLv2
"""
import string
import os.path
import logging
import re

import sx
from sx.reports import Report
from sx.reports.sosreport import SReport
from sx.reports.sosreport import DistroRelease
from sx.tarball import TarBall

class Sysreport(sx.reports.Report, SReport) :
    """
    This class is a container for sysreport report object. This is a
    child class of the SReport object that contains various functions
    that sosreport/sysreport use.

    @cvar TYPE_DETECTION_FILE: This is a path to file that can
    uniquely indentify the report object.
    @type TYPE_DETECTION_FILE: String
    """
    TYPE_DETECTION_FILE = "sysreport.log"
    def __init__(self) :
        sx.reports.Report.__init__(self, 
                                   "sysreport",
                                   "A report object for sysreport files")
        self.__hostname = "unknown_hostname"

    def extract(self, pathToReportFile, extractDir):
        """
        This function will extract the report to the extract
        dir. Currently all reports have to be a tarfile archive.

        This function overrides the parent function. The reason is
        that a new extractDir will be created. Then the parent extract
        function will be called.

        @return: Returns True if there was no fatal errors. TarBall
        errors are ignored because they should not be fatal, if they
        are the child should catch the errors since dir will not
        exist. If there are fatal errors, then False is returned.
        @rtype: boolean

        @param pathToReportFile: This is the path to the report file.
        @type pathToReportFile: String
        @param extractDir: The full path to directory for extraction.
        @type extractDir: String
        """
        fileContents = TarBall.getFileContents(pathToReportFile, "uname")
        if (len(fileContents) > 1):
            self.__hostname = string.strip(string.split(fileContents[1])[1])
        # Now call the parent function to finish the extraction
        sx.reports.Report.extract(self, pathToReportFile, 
                                  os.path.join(extractDir, self.__hostname))
        if (os.path.exists(os.path.join(self.getPathToExtractedReport(), 
                                        Sysreport.TYPE_DETECTION_FILE))):
            return True
        return False

    # ##########################################################################
    # Helper functions 
    # ##########################################################################
    def getUname(self) :
        """
        This function will return a string of the "uname -a" data.

        @return: Returns a string of the "uname -a" data.
        @rtype String
        """
        unameData = self.extractData("uname")
        if (unameData == None) :
            return ""
        return unameData[1].strip()

    def getUptime(self) :
        """
        This function will return a string of the "uptime" data.

        @return: Returns a string of the "uptime" data.
        @rtype String
        """
        uptime = self.extractData("uptime")
        if (uptime == None) :
            return ""
        return uptime[1].strip()

    def getArch(self) :
        """
        This function will return the arch for the report.

        @return: Returns the arch for the report.
        @rtype String
        """
        unameAData = self.getUname()
        regexArch = "(?P<arch>noarch|i386|i586|i686|ia64|ppc|s390|s390x|x86_64)" 
        remArch = re.compile(regexArch, re.IGNORECASE)
        moArch = remArch.search(unameAData)                                                                                                                       
        if moArch:    
            return moArch.group("arch")
        return ""

    def getHostname(self) :
        """
        This function will return a string that is the hostname from the
        "uname -a" data.

        @return: Returns a string that is the hostname from the "uname
        -a" data.
        @rtype String
        """
        if (not len(self.__hostname) > 0):
            # If hostname is not set or need from previous extracted report
            unameAData = self.getUname() 
            if (not len(unameAData) > 0) :
                self.__hostname = "unknown_hostname"
            else:
                usplit = unameAData.split()
                if (len(usplit) >= 1):
                    self.__hostname = usplit[1]
                else:
                    self.__hostname = "unknown_hostname"
        return self.__hostname 

    def getPathToInstalledRPMS(self) :
        """
        Returns the path to the file that contains the list of
        installed rpms.

        @returns: Returns the path to the file that contains the list
        of installed rpms.
        @rtype: String
        """
        pathToInstalledRPMS = self.extractPath("installed-rpms")
        return pathToInstalledRPMS

