#!/usr/bin/env python
"""
This file contains two classes. RPLoader is default loader and
PluginsLoader is a child of the RPLoader class. PluginsLoader is just
for plugins and not reports.

@author    :  Shane Bradley
@contact   :  sbradley@redhat.com
@version   :  2.03
@copyright :  GPLv2
"""
import string
import os
import os.path
import logging
import sys

import sx
from sx.logwriter import LogWriter

class RPLoader :

    def load(self, pathToPluginBaseDir, pluginImportBase):
        """
        This class will return a list of loaded(instance) objects based on path
        information. 

        @return: Returns a list of loaded(instance) objects based on
        path information.
        @rtype: Array

        @param pathToPluginBaseDir: The path to the base directory
        that contains the plugins.
        @type pathToPluginBaseDir: String
        @param pluginImportBase: The name of the plugins base 
        python import name.
        @type pluginImportBase: String
        """
        loadedPluginClasses = self.getClasses(pathToPluginBaseDir, pluginImportBase)
        loadedPlugins = []
        for pluginClass in loadedPluginClasses:
            if (not pluginClass == None):
                pluginInstance = pluginClass()
                loadedPlugins.append(pluginInstance)
        return loadedPlugins

    def getClasses(self, pathToPluginBaseDir, pluginImportBase):
        """
        This class will return a list of classes based on path
        information. 

        @return: Returns a list of classes based on path information.
        @rtype: Array

        @param pathToPluginBaseDir: The path to the base directory
        that contains the plugins.
        @type pathToPluginBaseDir: String
        @param pluginImportBase: The name of the plugins base 
        python import name.
        @type pluginImportBase: String
        """
        # List of plugins
        loadedPluginClasses = []

        if ((not len(pathToPluginBaseDir) > 0) or (not len(pluginImportBase) > 0)):
                return loadedPluginClasses

        # Set path to root of modules import
        pathToClassesDir = ("%s/%s") %(pathToPluginBaseDir, pluginImportBase.replace(".", "/"))

        # If the directory does not exist then nothing to do
        if (not (os.path.isdir(pathToClassesDir))):
            return loadedPluginClasses
         
        # Add path to modules if it does not exist in python path
        if (not (pathToPluginBaseDir in sys.path)) :
            sys.path.append(pathToPluginBaseDir)
            
        # Get list of files and load the classes into an array if they
        # are valid plugins
        filenames = os.listdir(pathToClassesDir)
        filenames.sort()
        # validate and load plugins
        for filename in filenames:
            pluginFilename =  filename[:-3]
            if ((not filename[-3:] == ".py") or (pluginFilename == "__init__")):
                continue
            if (self.__validatePlugin(os.path.join(pathToClassesDir,filename))) :
                try:
                    # First letter of plugin filename's class has to be be capitalized.
                    pluginClass = self.__importPlugin("%s.%s" %(pluginImportBase, pluginFilename), str.capitalize(pluginFilename))
                    loadedPluginClasses.append(pluginClass)
                except AttributeError:
                    message = "The class module was not found for %s." %(pluginFilename)
                    logging.getLogger(sx.MAIN_LOGGER_NAME).info(message)
            else:
                message = ("Plugin %s does not validate, skipping.") % (filename)
                logging.getLogger(sx.MAIN_LOGGER_NAME).info(message)
                continue
        return loadedPluginClasses

    def __validatePlugin(self, pluginPath) :
        """
        This is not implmented yet, but should verify that this is a
        plugin for sx and is valid.
        
        @return: Returns True if this valid sx plugin.
        @rtype: Boolean
        
        @param pluginPath: The path to the plugin.
        @type pluginPath: String
        """
        return True

    def __importPlugin(self, pathToPluginFile, pluginClassName):
        """
        This function will return a new class object based on the path and
        class name. None is returned if there is an error.
        
        @return: Returns a new class object based on path and class
        name. None is returned if there is an error.
        @rtype: Class
        
        @param pathToPluginFile: This is the path to plugin file that is in
        the python path.
        @type pathToPluginFile: String
        @param pluginClassName: Name of the class that will be imported
        @type pluginClassName: String
        """
        try:
            plugin = __import__(pathToPluginFile, globals(), locals(), [pluginClassName])
            return getattr(plugin, pluginClassName)
        except ValueError:
            pass
        except ImportError,e:
            message ="Import plugin error occurred on importing the Class \"%s\" from import: %s" %(pluginClassName, pathToPluginFile)
            logging.getLogger(sx.MAIN_LOGGER_NAME).error(message)
            print e
        return None

class PluginsLoader(RPLoader) :
    def load(self, pathToPluginBaseDir, pluginImportBase, pathToPluginReportDir):
        """
        This class will return a list of loaded(instance) objects based on path
        information. 

        @return: Returns a list of loaded(instance) objects based on
        path information.
        @rtype: Array

        @param pathToPluginBaseDir: The path to the base directory
        that contains the plugins.
        @type pathToPluginBaseDir: String
        @param pluginImportBase: The name of the plugins base 
        python import name.
        @type pluginImportBase: String
        @param pathToPluginReportDir: Path to the destination path
        that all reports will be extracted to.
        @type pathToPluginReportDir: String
        """
        loadedPluginClasses = self.getClasses(pathToPluginBaseDir, pluginImportBase)
        loadedPlugins = []
        for pluginClass in loadedPluginClasses:
            if (not pluginClass == None):
                pluginInstance = pluginClass(pathToPluginReportDir)
                loadedPlugins.append(pluginInstance)
        return loadedPlugins

    def mapPluginOptions(self, plugins, pluginOptionsMap):
        """
        The function will set the options for each plugin. 

        @param plugins: A list of plugins objects
        @type plugins: List
        @param pluginOptionsMap: A dictionary of options for 
        each plugin.
        @type pluginOptionsMap: Dictionary
        """
        for plugin in plugins:
            pluginName = plugin.getName().lower()
            if (pluginOptionsMap.has_key(pluginName)):
                optionsMap = pluginOptionsMap.get(pluginName)
                for optionName in optionsMap.keys():
                    plugin.setOptionValue(optionName, optionsMap.get(optionName))
                # Found the correct plugin so need to keep searching
                continue

    def setPluginsState(self, plugins, state):
        """
        This functions sets the state for a list of plugins. The
        states are True(enabled) or False(disabled).

        @param plugins: List of plugins.
        @type plugins: Array
        @param state: The state that will be set for the plugin.
        The states are True(enabled) or False(disabled).
        @type state: Boolean
        """
        for plugin in plugins:
            plugin.setEnabled(state)

