#!/usr/bin/env python
"""
Performs operations on a tarball that is gzipped or bzipped.  The
class uses python libs for inspection, but uses the sytem command for
tar to do the extraction.

@author    :  Shane Bradley
@contact   :  sbradley@redhat.com
@version   :  2.03
@copyright :  GPLv2
"""
import tarfile
import os.path
import string
import logging
import mimetypes
import subprocess

import sx
from sx.logwriter import LogWriter

class TarBall :
    """
    This class does extractions and queries on a tarball
    """
    def isCompressedTarFile(pathToTarFile) :
        """
        Returns True if the file is a compressed tar archive with
        compression type bzip2 or gunzip.

        @return: Returns True if the file is a compressed tar archive
        with compression type bzip2 or gunzip.
        @rtype: Boolean
        
        @param pathToTarFile: The path to the tarfile.
        @type pathToTarFile: String
        """
        # mimetype array indexes: (index 0 = file type), (index 1 = encoding type)
        mimeType = mimetypes.guess_type(pathToTarFile)
        if ((mimeType[0] == "application/x-tar") and 
            ((mimeType[1] == "gzip") or ((mimeType[1] == "bzip2")))) :
            try:
                # This is an extra check just to verify it is tarfile
                return tarfile.is_tarfile(pathToTarFile) 
            except TarError:
                message = "There was an error checking if file was a tarfile: %s" %(pathToTarFile)
                logging.getLogger(sx.MAIN_LOGGER_NAME).error(message)
        return False
    isCompressedTarFile = staticmethod(isCompressedTarFile)

    def getCompressionType(pathToTarFile) :
        """
        Returns the type of compression that is used on the
        tarfile. The file has to be a tarfile or "" is returned. It is
        either gunzip, bz2, "". Where empty string in unknown.

        @return: Returns the type of compression that is used on
        tarfile.
        @rtype: String
        
        @param pathToTarFile: The path to the tarfile.
        @type pathToTarFile: String
        """
        if (TarBall.isCompressedTarFile(pathToTarFile)) :
            return mimetypes.guess_type(pathToTarFile)[1]
        return ""
    getCompressionType = staticmethod(getCompressionType)

    def getExtactOptions(pathToTarFile) :
        """
        This function returns the command options for extracting a
        tarball that is either gunzipped or BZ2 zipped. It gets the
        options for GNU tar command. Empty String is returned if
        unknown.

        @return: Returns string that is the option to extract the
        tarball.
        @rtype: String

        @param pathToTarFile: Fullpath to the file that will be extracted.
        @type pathToTarFile: String
        """
        cType = TarBall.getCompressionType(pathToTarFile) 
        if (cType == "gzip"):
            return "zxpf"
        elif (cType == "bzip2") :
            return "jxpf"
        return ""
    getExtactOptions = staticmethod(getExtactOptions)

    def getTarMode(pathToTarFile) :
        """
        This function returns the mode that can be used with python's
        tarfile module. Empty string is returned if unknown.

        @return: Returns string that is the mode for the tarfile.
        @rtype: String

        @param pathToTarFile: Fullpath to the file that will be extracted.
        @type pathToTarFile: String
        """
        cType = TarBall.getCompressionType(pathToTarFile) 
        if (cType == "gzip"):
            return "gz"
        elif (cType == "bzip2") :
            return "bz2"
        return ""
    getTarMode = staticmethod(getTarMode)
        
    def getFileContents(pathToTarFile, pathToFile) :
        """
        The function returns an array of all the filecontents in the file.

        @return: Returns the first match to a path in tar achive file
        contents which is an array of strings.
        @rtype: Array

        @param pathToTarFile: Path to the tar archive.
        @type pathToTarFile: String
        @param pathToFile: Path to file or subset of path to file in
        the tararchive.
        @type pathToFile: String
        """
        if (not TarBall.isCompressedTarFile(pathToTarFile)) :
            return []
        mode = TarBall.getTarMode(pathToTarFile)
        if (len(mode) > 0) :
            try:
                tar = tarfile.open(pathToTarFile, "r:%s" % mode)
                members = tar.getmembers() 
                tarinfo = None
                for currentTarInfo in members: 
                    if (currentTarInfo.name.find(pathToFile) >= 0) :
                        tarinfo = currentTarInfo
                        break
                if tarinfo == None:
                    return ""
                file = tar.extractfile(tarinfo)
                data = file.readlines() 
                file.close
                tar.close()
                return data
            except (IOError, os.error, KeyError):
                message = "There was a problem with extracting the tarfile and getting file %s." %(pathToFile)
                logging.getLogger(sx.MAIN_LOGGER_NAME).error(message)
                tar.close()
        return []
    getFileContents = staticmethod(getFileContents)

    def list(pathToTarFile) :
        """
        This function will return an array of the file paths in the tarfile.

        @return: Returns an array of all the file paths in the
        tarfile. Empty array is returned if there is an error.
        @rtype: Array

        @param pathToTarFile: Fullpath to the file that will be extracted.
        @type pathToTarFile: String
        """
        try:
            mode = TarBall.getTarMode(pathToTarFile)
            if (not len(mode) > 0):
                message =  "The tarfile is an invalid or unrecongized type: %s" %(pathToTarFile)
                logging.getLogger(sx.MAIN_LOGGER_NAME).debug(message)
                return []
            tar = tarfile.open(pathToTarFile, "r:%s" % mode)
            data = tar.getnames()
            tar.close()
            return data
        except (IOError, os.error):
            message = "There was an IO error retreiving the list of files in the tar file: %s" %(pathToTarFile)
            logging.getLogger(sx.MAIN_LOGGER_NAME).error(message)
            tar.close()
        except (EOFError):
            message = "There was an EOF error retreiving the list of files in the tar file: %s" %(pathToTarFile)
            logging.getLogger(sx.MAIN_LOGGER_NAME).error(message)
        return []
    list = staticmethod(list)

    def extract(extractDir, pathToTarFile, stripDirectoriesDepth=1) :
        """
        This command extracts a tarball to a specific location. This
        uses GNU Tar to extract the file.

        @return: True if the function did not throw an exception.
        @rtype: Boolean

        @param extractDir: The directory that tarball will be
        extracted to.
        @type extractDir: String
        @param pathToTarFile: Fullpath to the file that will be extracted.
        @type pathToTarFile: String
        @param stripDirectoriesDepth: This is int for the number of 
        directories to strip off of the extracted file. Default
        is 1.
        @type stripDirectoriesDepth: Int
        """
        # Path to the command that will be used
        pathToTarCommand = "tar"
        # The extraction mode that will be used
        mode = TarBall.getTarMode(pathToTarFile)
        # The command options that will be used
        commandOptions = TarBall.getExtactOptions(pathToTarFile)

        # Verify that this is GNU version
        command = [pathToTarCommand, "--version"]
        tarTask = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        (stdout, stderr) = tarTask.communicate()
        if (not stdout.find("GNU") >= 0):
            message = "The tar command does not appear to be GNU version: %s" %(pathToTarCommand)
            logging.getLogger(sx.MAIN_LOGGER_NAME).error(message)
            return False
        elif (not len(mode) > 0):
            message =  "The tarfile is an invalid or unrecongized type: \n\t%s" %(pathToTarFile)
            logging.getLogger(sx.MAIN_LOGGER_NAME).error(message)
            return False
        elif (not len(commandOptions) > 0) :
            message =  "This file is unknown type and will not be extracted: %s" %(pathToTarFile)
            logging.getLogger(sx.MAIN_LOGGER_NAME).error(message)
            return False
        else:
            # The tarfile must be valid so it will be extracted.
            # ###################################################################
            # Run the command: tar -jxvf blah.tar.bz2 -C /usr/lib/blah/ --strip-components 1 
            # Note: I remove the quote of path because command did not like it.
            # ###################################################################
            command = [pathToTarCommand, commandOptions, pathToTarFile, "-C", extractDir, "--strip-components", str(stripDirectoriesDepth)]
            tarTask = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            (stdout, stderr) = tarTask.communicate()
            if (not tarTask.returncode  == 0):
                message = "There was an error extracting tarball with GNU tar on file:\n\t  %s" % (pathToTarFile)
                logging.getLogger(sx.MAIN_LOGGER_NAME).debug(message)
            return os.path.isdir(extractDir)
    extract = staticmethod(extract)
        
